@@ -36,11 +36,15 @@ class ParseOptions(object):
                  islands_ok=False,
                  short_length=16,
                  all_short_connectors=False,
-                 display_morphology=False,
+                 display_morphology=True,
                  spell_guess=False,
                  use_sat=False,
                  max_parse_time=-1,
-                 disjunct_cost=2.7):
+                 disjunct_cost=2.7,
+                 repeatable_rand=True,
+                 test='',
+                 debug='',
+                 ):
 
         self._obj = clg.parse_options_create()
         self.verbosity = verbosity
@@ -55,19 +59,42 @@ class ParseOptions(object):
         self.use_sat = use_sat
         self.max_parse_time = max_parse_time
         self.disjunct_cost = disjunct_cost
+        self.repeatable_rand = repeatable_rand
+        self.test = test
+        self.debug = debug
 
     # Allow only the attribute names listed below.
     def __setattr__(self, name, value):
         if not hasattr(self, name) and name != "_obj":
             # TypeError for consistency. It maybe should have been NameError.
             raise TypeError('Unknown parse option "{}".'.format(name))
-        super(self.__class__, self).__setattr__(name, value)
+        super(ParseOptions, self).__setattr__(name, value)
 
     def __del__(self):
         if hasattr(self, '_obj'):
             clg.parse_options_delete(self._obj)
             del self._obj
 
+    @property
+    def test(self):
+        return clg.parse_options_get_test(self._obj)
+
+    @test.setter
+    def test(self, value):
+        if not isinstance(value, str):
+            raise TypeError("islands_ok must be set to a string")
+        return clg.parse_options_set_test(self._obj, value)
+
+    @property
+    def debug(self):
+        return clg.parse_options_get_debug(self._obj)
+
+    @debug.setter
+    def debug(self, value):
+        if not isinstance(value, str):
+            raise TypeError("islands_ok must be set to a string")
+        return clg.parse_options_set_debug(self._obj, value)
+
     @property
     def verbosity(self):
         """
@@ -274,6 +301,22 @@ class ParseOptions(object):
             raise TypeError("all_short_connectors must be set to a bool")
         clg.parse_options_set_all_short_connectors(self._obj, 1 if value else 0)
 
+    @property
+    def repeatable_rand(self):
+        """
+        If set to True, then a repeatable random sequence will be used, whenever
+        a random number is required.  The parser almost never uses random
+        numbers; currently they are only used in one place: to sample a subset
+        of linkages, if there are more parses than linkage_limit.
+        """
+        return clg.parse_options_get_repeatable_rand(self._obj) == 1
+
+    @repeatable_rand.setter
+    def repeatable_rand(self, value):
+        if not isinstance(value, bool):
+            raise TypeError("repeatable_rand must be set to a bool")
+        clg.parse_options_set_repeatable_rand(self._obj, 1 if value else 0)
+
 
 class LG_Error(Exception):
     @staticmethod
@@ -483,11 +526,13 @@ class Sentence(object):
     def __init__(self, text, lgdict, parse_options):
         # Keep all args passed into clg.* functions.
         self.text, self.dict, self.parse_options = text, lgdict, parse_options
+        clg._py_incref(self.dict) # The Sentence struct refers to the Dictionary struct
         self._obj = clg.sentence_create(self.text, self.dict._obj)
 
     def __del__(self):
         if hasattr(self, '_obj'):
             clg.sentence_delete(self._obj)
+            clg._py_decref(self.dict)
             del self._obj
 
     def split(self, parse_options=None):
