@@ -15,8 +15,8 @@
 
 #include "api-structures.h"
 #include "corpus/corpus.h"
-#include "dict-common/dict-utils.h" // for free_X_nodes
-#include "disjunct-utils.h"  // for free_disjuncts
+#include "dict-common/dict-utils.h"
+#include "disjunct-utils.h"   // for free_sentence_disjuncts()
 #include "linkage/linkage.h"
 #include "memory-pool.h"
 #include "parse/histogram.h"  // for PARSE_NUM_OVERFLOW
@@ -28,7 +28,7 @@
 #include "sat-solver/sat-encoder.h"
 #include "tokenize/spellcheck.h"
 #include "tokenize/tokenize.h"
-#include "tokenize/word-structures.h" // Needed for Word_struct/free_X_node
+#include "tokenize/word-structures.h" // Needed for Word_struct
 #include "utilities.h"
 
 /* Its OK if this is racey across threads.  Any mild shuffling is enough. */
@@ -55,9 +55,6 @@ static int VDAL_compare_parse(Linkage l1, Linkage l2)
 	Linkage_info * p1 = &l1->lifo;
 	Linkage_info * p2 = &l2->lifo;
 
-	/* Move the discarded entries to the end of the list */
-	if (p1->discarded || p2->discarded) return (p1->discarded - p2->discarded);
-
 	if (p1->N_violations != p2->N_violations) {
 		return (p1->N_violations - p2->N_violations);
 	}
@@ -79,9 +76,6 @@ static int CORP_compare_parse(Linkage l1, Linkage l2)
 
 	double diff = p1->corpus_cost - p2->corpus_cost;
 
-	/* Move the discarded entries to the end of the list */
-	if (p1->discarded || p2->discarded) return (p1->discarded - p2->discarded);
-
 	if (fabs(diff) < 1.0e-5)
 		return VDAL_compare_parse(p1, p2);
 	if (diff < 0.0) return -1;
@@ -442,12 +436,29 @@ Sentence sentence_create(const char *input_string, Dictionary dict)
 	sent->dict = dict;
 	sent->string_set = string_set_create();
 	sent->rand_state = global_rand_state;
+	sent->E_list_pool = pool_new(__func__, "E_list", /*num_elements*/4096,
+	                             sizeof(E_list), /*zero_out*/false,
+	                             /*align*/false, /*exact*/false);
+	sent->Exp_pool = pool_new(__func__, "Exp", /*num_elements*/4096,
+	                             sizeof(Exp), /*zero_out*/false,
+	                             /*align*/false, /*exact*/false);
+	sent->X_node_pool = pool_new(__func__, "X_node", /*num_elements*/256,
+	                             sizeof(X_node), /*zero_out*/false,
+	                             /*align*/false, /*exact*/false);
 
 	sent->postprocessor = post_process_new(dict->base_knowledge);
 
 	/* Make a copy of the input */
 	sent->orig_sentence = string_set_add (input_string, sent->string_set);
 
+	/* Set the minimum length for tracon sharing.
+	 * In that case a tracon list is produced for the pruning step,
+	 * and disjunct/connector packing is done too. */
+	sent->min_len_encoding = SENTENCE_MIN_LENGTH_TRAILING_HASH;
+	const char *min_len_encoding = test_enabled("len-trailing-hash");
+	if (NULL != min_len_encoding)
+		sent->min_len_encoding = atoi(min_len_encoding+1);
+
 	return sent;
 }
 
@@ -501,7 +512,6 @@ static void free_sentence_words(Sentence sent)
 {
 	for (WordIdx i = 0; i < sent->length; i++)
 	{
-		free_X_nodes(sent->word[i].x);
 		free(sent->word[i].alternatives);
 	}
 	free_sentence_disjuncts(sent);
@@ -516,7 +526,6 @@ void sentence_delete(Sentence sent)
 	free_sentence_words(sent);
 	wordgraph_delete(sent);
 	string_set_delete(sent->string_set);
-	string_id_delete(sent->connector_suffix_id);
 	free_linkages(sent);
 	post_process_free(sent->postprocessor);
 	post_process_free(sent->constituent_pp);
@@ -524,6 +533,9 @@ void sentence_delete(Sentence sent)
 	global_rand_state = sent->rand_state;
 	pool_delete(sent->fm_Match_node);
 	pool_delete(sent->Table_connector_pool);
+	pool_delete(sent->E_list_pool);
+	pool_delete(sent->Exp_pool);
+	pool_delete(sent->X_node_pool);
 	if (IS_DB_DICT(sent->dict))
 		condesc_reuse(sent->dict);
 
