@@ -68,27 +68,38 @@ int size_of_expression(Exp * e)
 /**
  * Build a copy of the given expression (don't copy strings, of course)
  */
-static E_list * copy_E_list(E_list * l);
-Exp * copy_Exp(Exp * e)
+static E_list *copy_E_list(E_list *l, Pool_desc* mp[])
 {
-	Exp * n;
-	if (e == NULL) return NULL;
-	n = malloc(sizeof(Exp));
-	*n = *e;
-	if (e->type != CONNECTOR_type) {
-		n->u.l = copy_E_list(e->u.l);
+
+	E_list el_head;
+	E_list *el = &el_head;
+
+	for (; l != NULL; l = l->next)
+	{
+		E_list *nl = pool_alloc(mp[0]);
+
+		nl->e = pool_alloc(mp[1]);
+		*nl->e = *l->e;
+		el->next = nl;
+		el = nl;
+		if (l->e->type != CONNECTOR_type)
+			nl->e->u.l = copy_E_list(l->e->u.l, mp);
 	}
-	return n;
+	el->next = NULL;
+
+	return el_head.next;
 }
 
-static E_list * copy_E_list(E_list * l)
+Exp *copy_Exp(Exp *e, Pool_desc *E_list_pool, Pool_desc *Exp_pool)
 {
-	E_list * nl;
-	if (l == NULL) return NULL;
-	nl = malloc(sizeof(E_list));
-	nl->next = copy_E_list(l->next);
-	nl->e = copy_Exp(l->e);
-	return nl;
+	if (e == NULL) return NULL;
+	Exp *ne = pool_alloc(Exp_pool);
+
+	*ne = *e;
+	if (CONNECTOR_type == e->type) return ne;
+
+	ne->u.l = copy_E_list(ne->u.l, (Pool_desc*[]){E_list_pool, Exp_pool});
+	return ne;
 }
 
 /**
@@ -165,18 +176,6 @@ static int exp_contains(Exp * super, Exp * sub)
 
 /* ======================================================== */
 /* X_node utilities ... */
-/**
- * frees the list of X_nodes pointed to by x, and all of the expressions
- */
-void free_X_nodes(X_node * x)
-{
-	X_node * y;
-	for (; x!= NULL; x = y) {
-		y = x->next;
-		free_Exp(x->exp);
-		free(x);
-	}
-}
 
 /**
  * Destructively catenates the two disjunct lists d1 followed by d2.
@@ -194,6 +193,17 @@ X_node * catenate_X_nodes(X_node *d1, X_node *d2)
 	return d1;
 }
 
+#ifdef DEBUG
+GNUC_UNUSED static void print_x_node(X_node *x)
+{
+	if (x == NULL) printf("NULL X_node\n");
+	for (; x != NULL; x = x->next)
+	{
+		printf("%p: exp=%p next=%p\n", x, x->exp, x->next);
+	}
+}
+#endif
+
 /* ======================================================== */
 /* More connector utilities ... */
 
