@@ -13,12 +13,12 @@
 
 #include <string.h>
 
+#include "api-structures.h"           // For Sentence_s (add_empty_word())
 #include "dict-common/dict-affix.h"   // For is_stem()
 #include "dict-common/dict-common.h"
 #include "dict-common/dict-defines.h" // For SUBSCRIPT_MARK
 #include "dict-common/file-utils.h"
 #include "dict-common/idiom.h"
-#include "dict-common/regex-morph.h"
 #include "error.h"
 #include "print/print.h"
 #include "externs.h"
@@ -276,6 +276,7 @@ static bool char_is_special(char c)
  * This reads the next token from the input into 'token'.
  * Return 1 if a character was read, else return 0 (and print a warning).
  */
+NO_SAN_DICT
 static bool link_advance(Dictionary dict)
 {
 	utf8char c;
@@ -489,6 +490,7 @@ int dict_order_strict(char *s, char *t)
 
 /* terse version */
 /* If one word contains a dot, the other one must also! */
+NO_SAN_DICT
 static inline int dict_order_strict(const char *s, const Dict_node * dn)
 {
 	const char * t = dn->string;
@@ -511,7 +513,7 @@ static inline int dict_order_strict(const char *s, const Dict_node * dn)
  * If the dictionary string contains a SUBSCRIPT_MARK, then replace the
  * mark by "\0", and take the difference.
  */
-
+NO_SAN_DICT
 static inline int dict_order_bare(const char *s, const Dict_node * dn)
 {
 	const char * t = dn->string;
@@ -589,7 +591,7 @@ static inline Dict_node * dict_node_new(void)
  * not match the dictionary string s. The matching is dictionary matching:
  * subscripted entries will match "bare" entries.
  */
-static Dict_node * prune_lookup_list(Dict_node *llist, const char * s)
+static Dict_node * prune_lookup_list(Dict_node * restrict llist, const char * restrict s)
 {
 	Dict_node *dn, *dnx, *list_new;
 
@@ -641,9 +643,9 @@ static bool subscr_match(const char *s, const Dict_node * dn)
  * make a copy of that node, and append it to llist.
  */
 static Dict_node *
-rdictionary_lookup(Dict_node *llist,
-                   const Dict_node * dn,
-                   const char * s,
+rdictionary_lookup(Dict_node * restrict llist,
+                   const Dict_node * restrict dn,
+                   const char * restrict s,
                    bool match_idiom,
                    int (*dict_order)(const char *, const Dict_node *))
 {
@@ -853,6 +855,26 @@ static Exp * make_and_node(Exp_list * eli, Exp* nl, Exp* nr)
 	return n;
 }
 
+static Exp *make_op_Exp(Exp_list *eli, Exp_type t)
+{
+	Exp * n = Exp_create(eli);
+	n->type = t;
+	n->cost = 0.0;
+	n->u.l = NULL;
+
+	return n;
+}
+
+static E_list *make_E_list_val(Exp* nr)
+{
+	E_list *elr = (E_list *) malloc(sizeof(E_list));
+
+	elr->e = nr;
+	elr->next = NULL;
+
+	return elr;
+}
+
 /**
  * Create an OR_type expression. The expressions nl, nr will be
  * OR-ed together.
@@ -929,7 +951,7 @@ static Exp * make_connector(Dictionary dict)
 	Dict_node *dn, *dn_head;
 	int i;
 
-	i = strlen(dict->token) - 1;  /* this must be +, - or * if a connector */
+	i = strlen(dict->token) - 1;  /* this must be +, - or $ if a connector */
 	if ((dict->token[i] != '+') &&
 	    (dict->token[i] != '-') &&
 	    (dict->token[i] != ANY_DIR))
@@ -950,7 +972,10 @@ static Exp * make_connector(Dictionary dict)
 			                 "Or perhaps a word is used before it is defined.");
 			return NULL;
 		}
+
+		/* Wrap it in a unary node as a placeholder for a cost if needed. */
 		n = make_unary_node(&dict->exp_list, dn->exp);
+
 		file_free_lookup(dn_head);
 	}
 	else
@@ -998,17 +1023,53 @@ static Exp * make_connector(Dictionary dict)
 /* ======================================================================== */
 /* Empty-word handling. */
 
+/** Expression-creating function versions that use the Sentence
+ *  expression memory pools.
+ *  (See the description of the similar functions above.)
+ */
+static Exp *make_or_node_from_pool(Sentence sent, Exp *nl, Exp *nr)
+{
+	E_list *ell, *elr;
+	Exp* n;
+
+	n = pool_alloc(sent->Exp_pool);
+	n->type = OR_type;
+	n->cost = 0.0;
+
+	n->u.l = ell = (E_list *) pool_alloc(sent->E_list_pool);
+	ell->next = elr = (E_list *) pool_alloc(sent->E_list_pool);
+	elr->next = NULL;
+
+	ell->e = nl;
+	elr->e = nr;
+	return n;
+}
+
+static Exp *make_zeroary_node_from_pool(Sentence sent)
+{
+	Exp * n = pool_alloc(sent->Exp_pool);
+	n->type = AND_type;  /* these must be AND types */
+	n->cost = 0.0;
+	n->u.l = NULL;
+	return n;
+}
+
+static Exp *make_optional_node_from_pool(Sentence sent, Exp *e)
+{
+	return make_or_node_from_pool(sent, make_zeroary_node_from_pool(sent), e);
+}
+
 /** Insert ZZZ+ connectors.
  *  This function was mainly used to support using empty-words, a concept
  *  that has been eliminated. However, it is still used to support linking of
  *  quotes that don't get the QUc/QUd links.
  */
-void add_empty_word(Dictionary const dict, X_node *x)
+void add_empty_word(Sentence sent, X_node *x)
 {
 	Exp *zn, *an;
 	E_list *elist, *flist;
-	Exp_list eli = { NULL };
-	const char *ZZZ = string_set_add(EMPTY_CONNECTOR, dict->string_set);
+	const char *ZZZ = string_set_lookup(EMPTY_CONNECTOR, sent->dict->string_set);
+	/* This function is called only if ZZZ is in the dictionary. */
 
 	/* The left-wall already has ZZZ-. The right-wall will not arrive here. */
 	if (MT_WALL == x->word->morpheme_type) return;
@@ -1022,26 +1083,26 @@ void add_empty_word(Dictionary const dict, X_node *x)
 		//lgdebug(+0, "Processing '%s'\n", x->string);
 
 		/* zn points at {ZZZ+} */
-		zn = Exp_create(&eli);
+		zn = pool_alloc(sent->Exp_pool);
 		zn->dir = '+';
-		zn->u.condesc = condesc_add(&dict->contable, ZZZ);
+		zn->u.condesc = condesc_add(&sent->dict->contable, ZZZ);
 		zn->multi = false;
 		zn->type = CONNECTOR_type;
 		zn->cost = 0.0;
-		zn = make_optional_node(&eli, zn);
+		zn = make_optional_node_from_pool(sent, zn);
 
 		/* flist is plain-word-exp */
-		flist = (E_list *) malloc(sizeof(E_list));
+		flist = (E_list *) pool_alloc(sent->E_list_pool);
 		flist->next = NULL;
 		flist->e = x->exp;
 
 		/* elist is {ZZZ+} , (plain-word-exp) */
-		elist = (E_list *) malloc(sizeof(E_list));
+		elist = (E_list *) pool_alloc(sent->E_list_pool);
 		elist->next = flist;
 		elist->e = zn;
 
 		/* an will be {ZZZ+} & (plain-word-exp) */
-		an = Exp_create(&eli);
+		an = pool_alloc(sent->Exp_pool);
 		an->type = AND_type;
 		an->cost = 0.0;
 		an->u.l = elist;
@@ -1189,178 +1250,195 @@ static Exp * expression(Dictionary dict)
 /* ======================================================================== */
 #else /* This is for infix notation */
 
-static Exp * restricted_expression(Dictionary dict, int and_ok, int or_ok);
-
 /**
  * Build (and return the root of) the tree for the expression beginning
  * with the current token.  At the end, the token is the first one not
  * part of this expression.
  */
 static Exp * make_expression(Dictionary dict)
-{
-	return restricted_expression(dict, true, true);
-}
-
-static Exp * restricted_expression(Dictionary dict, int and_ok, int or_ok)
 {
 	Exp *nl = NULL;
+	Exp *e_head = NULL;
+	E_list *el_tail = NULL; /* last part of the expression */
+	bool is_sym_and = false;
 
-	if (is_equal(dict, '('))
+	while (true)
 	{
-		if (!link_advance(dict)) {
-			return NULL;
-		}
-		nl = make_expression(dict);
-		if (nl == NULL) {
-			return NULL;
-		}
-		if (!is_equal(dict, ')')) {
-			dict_error(dict, "Expecting a \")\".");
-			return NULL;
-		}
-		if (!link_advance(dict)) {
-			return NULL;
-		}
-	}
-	else if (is_equal(dict, '{'))
-	{
-		if (!link_advance(dict)) {
-			return NULL;
-		}
-		nl = make_expression(dict);
-		if (nl == NULL) {
-			return NULL;
-		}
-		if (!is_equal(dict, '}')) {
-			dict_error(dict, "Expecting a \"}\".");
-			return NULL;
-		}
-		if (!link_advance(dict)) {
-			return NULL;
-		}
-		nl = make_optional_node(&dict->exp_list, nl);
-	}
-	else if (is_equal(dict, '['))
-	{
-		if (!link_advance(dict)) {
-			return NULL;
-		}
-		nl = make_expression(dict);
-		if (nl == NULL) {
-			return NULL;
-		}
-		if (!is_equal(dict, ']')) {
-			dict_error(dict, "Expecting a \"]\".");
-			return NULL;
-		}
-		if (!link_advance(dict)) {
-			return NULL;
-		}
-
-		/* A square bracket can have a number after it.
-		 * If it is present, then that number is interpreted
-		 * as the cost of the bracket. Else, the cost of a
-		 * square bracket is 1.0.
-		 */
-		if (is_number(dict->token))
+		if (is_equal(dict, '('))
 		{
-			double cost;
-			if (strtodC(dict->token, &cost))
-			{
-				nl->cost += cost;
+			if (!link_advance(dict)) {
+				return NULL;
 			}
-			else
-			{
-				warning(dict, "Invalid cost (using 1.0)\n");
-				nl->cost += 1.0;
+			nl = make_expression(dict);
+			if (nl == NULL) {
+				return NULL;
+			}
+			if (!is_equal(dict, ')')) {
+				dict_error(dict, "Expecting a \")\".");
+				return NULL;
 			}
 			if (!link_advance(dict)) {
 				return NULL;
 			}
 		}
+		else if (is_equal(dict, '{'))
+		{
+			if (!link_advance(dict)) {
+				return NULL;
+			}
+			nl = make_expression(dict);
+			if (nl == NULL) {
+				return NULL;
+			}
+			if (!is_equal(dict, '}')) {
+				dict_error(dict, "Expecting a \"}\".");
+				return NULL;
+			}
+			if (!link_advance(dict)) {
+				return NULL;
+			}
+			nl = make_optional_node(&dict->exp_list, nl);
+		}
+		else if (is_equal(dict, '['))
+		{
+			if (!link_advance(dict)) {
+				return NULL;
+			}
+			nl = make_expression(dict);
+			if (nl == NULL) {
+				return NULL;
+			}
+			if (!is_equal(dict, ']')) {
+				dict_error(dict, "Expecting a \"]\".");
+				return NULL;
+			}
+			if (!link_advance(dict)) {
+				return NULL;
+			}
+
+			/* A square bracket can have a number after it.
+			 * If it is present, then that number is interpreted
+			 * as the cost of the bracket. Else, the cost of a
+			 * square bracket is 1.0.
+			 */
+			if (is_number(dict->token))
+			{
+				double cost;
+
+				if (strtodC(dict->token, &cost))
+				{
+					nl->cost += cost;
+				}
+				else
+				{
+					warning(dict, "Invalid cost (using 1.0)\n");
+					nl->cost += 1.0;
+				}
+				if (!link_advance(dict)) {
+					return NULL;
+				}
+			}
+			else
+			{
+				nl->cost += 1.0;
+			}
+		}
+		else if (!dict->is_special)
+		{
+			nl = make_connector(dict);
+			if (nl == NULL) {
+				return NULL;
+			}
+		}
+		else if (is_equal(dict, ')') || is_equal(dict, ']'))
+		{
+			/* allows "()" or "[]" */
+			nl = make_zeroary_node(&dict->exp_list);
+		}
 		else
 		{
-			nl->cost += 1.0;
-		}
-	}
-	else if (!dict->is_special)
-	{
-		nl = make_connector(dict);
-		if (nl == NULL) {
+			dict_error(dict, "Connector, \"(\", \"[\", or \"{\" expected.");
 			return NULL;
 		}
-	}
-	else if (is_equal(dict, ')') || is_equal(dict, ']'))
-	{
-		/* allows "()" or "[]" */
-		nl = make_zeroary_node(&dict->exp_list);
-	}
-	else
-	{
-		dict_error(dict, "Connector, \"(\", \"[\", or \"{\" expected.");
-		return NULL;
-	}
 
-	/* Non-commuting AND */
-	if (is_equal(dict, '&') || (strcmp(dict->token, "and") == 0))
-	{
-		Exp *nr;
+		if (is_sym_and)
+		{
+			/* Part 2/2 of SYM_AND processing */
+			E_list **elp; /* LHS argument and the SYM_AND result placeholder */
+			Exp *nr, *na, *nb, *or;
 
-		if (!and_ok) {
-			warning(dict, "\"and\" and \"or\" at the same level in an expression");
+			elp = (el_tail == NULL) ? &e_head->u.l : &el_tail;
+			nr = nl; /* last expression is actually the RHS */
+			nl = (*elp)->e;
+
+			/* Expand A ^ B into the expr ((A & B) or (B & A)).
+			 * It can be mixed with ordinary ands at the same level. */
+			na = make_and_node(&dict->exp_list, nl, nr);
+			nb = make_and_node(&dict->exp_list, nr, nl);
+			or = make_or_node(&dict->exp_list, na, nb);
+
+			(*elp)->e = or;
+			is_sym_and = false;
 		}
+		else if (el_tail != NULL)
+		{
+			/* This is Non-commuting AND or Commuting OR.
+			 * Append the just read expression (nl) to its E_list chain. */
+			el_tail->next = make_E_list_val(nl);
+			el_tail = el_tail->next;
+		}
+
+		/* If (el_tail == NULL), handle_basic_op() below will create an
+		 * E_list element with nl. Else it will just validate that the
+		 * current operation is consistent with the current expression level. */
+
+		Exp_type op;
+
+		/* Non-commuting AND */
+		if (is_equal(dict, '&') || (strcmp(dict->token, "and") == 0))
+		{
+			op = AND_type;
+		}
+		/* Commuting OR */
+		else if (is_equal(dict, '|') || (strcmp(dict->token, "or") == 0))
+		{
+			op =  OR_type;
+		}
+		/* Commuting AND */
+		else if (is_equal(dict, SYM_AND) || (strcmp(dict->token, "sym") == 0))
+		{
+			/* Part 1/2 of SYM_AND processing */
+			op = AND_type;
+			is_sym_and = true; /* processing to be completed after next argument */
+		}
+		else
+		{
+			if (e_head != NULL) return e_head;
+			return nl;
+		}
+
+		if (e_head != NULL)
+		{
+			if (e_head->type != op)
+			{
+				dict_error(dict, "\"and\" and \"or\" at the same level in an expression");
+				return NULL;
+			}
+		}
+		else
+		{
+			e_head = make_op_Exp(&dict->exp_list, op);
+			e_head->u.l = make_E_list_val(nl);
+		}
+
 		if (!link_advance(dict)) {
 			return NULL;
 		}
-		nr = restricted_expression(dict, true, false);
-		if (nr == NULL) {
-			return NULL;
-		}
-		return make_and_node(&dict->exp_list, nl, nr);
+
+		if (el_tail == NULL)
+			el_tail = e_head->u.l;
 	}
-	/* Commuting OR */
-	else if (is_equal(dict, '|') || (strcmp(dict->token, "or") == 0))
-	{
-		Exp *nr;
-
-		if (!or_ok) {
-			warning(dict, "\"and\" and \"or\" at the same level in an expression");
-		}
-		if (!link_advance(dict)) {
-			return NULL;
-		}
-		nr = restricted_expression(dict, false, true);
-		if (nr == NULL) {
-			return NULL;
-		}
-		return make_or_node(&dict->exp_list, nl, nr);
-	}
-	/* Commuting AND */
-	else if (is_equal(dict, SYM_AND) || (strcmp(dict->token, "sym") == 0))
-	{
-		Exp *nr, *na, *nb, *or;
-
-		if (!and_ok) {
-			warning(dict, "\"and\" and \"or\" at the same level in an expression");
-		}
-		if (!link_advance(dict)) {
-			return NULL;
-		}
-		nr = restricted_expression(dict, true, false);
-		if (nr == NULL) {
-			return NULL;
-		}
-
-		/* Expand A ^ B into the expr ((A & B) or (B & A)).
-		 * Must be wrapped in unary node so that it can be
-		 * mixed with ordinary ands at the same level. */
-		na = make_and_node(&dict->exp_list, nl, nr);
-		nb = make_and_node(&dict->exp_list, nr, nl);
-		or = make_or_node(&dict->exp_list, na, nb);
-		return make_unary_node(&dict->exp_list, or);
-	}
-
-	return nl;
+		/* unreachable */
 }
 
 #endif
@@ -1419,6 +1497,7 @@ static Dict_node * dsw_tree_to_vine (Dict_node *root)
 	return vh.right;
 }
 
+NO_SAN_DICT
 static void dsw_compression (Dict_node *root, unsigned int count)
 {
 	unsigned int j;
@@ -1469,6 +1548,7 @@ static Dict_node * dsw_vine_to_tree (Dict_node *root, int size)
  * The resulting tree is highly unbalanced. It needs to be rebalanced
  * before being used.  The DSW algo below is ideal for that.
  */
+NO_SAN_DICT
 Dict_node * insert_dict(Dictionary dict, Dict_node * n, Dict_node * newnode)
 {
 	if (NULL == n) return newnode;
@@ -1732,14 +1812,16 @@ static bool read_entry(Dictionary dict)
 			instr = get_file_contents(dict_name + skip_slash);
 			if (NULL == instr)
 			{
-				prt_error("Error: Could not open subdictionary \"%s\"\n", dict_name);
+				prt_error("Error: While parsing dictionary \"%s\":\n"
+				          "\t Line %d: Could not open subdictionary \"%s\"\n",
+				          dict->name, dict->line_number-1, dict_name);
 				goto syntax_error;
 			}
 			dict->input = instr;
 			dict->pin = dict->input;
 
 			/* The line number and dict name are used for error reporting */
-			dict->line_number = 0;
+			dict->line_number = 1;
 			dict->name = dict_name;
 
 			/* Now read the thing in. */
