@@ -2,7 +2,7 @@
  * read-sql.c
  *
  * Look up words in an SQL DB.
- * Keeping it simple for just right now, and using SQLite.
+ * Keeping it simple for just right now, and using SQLite3.
  *
  * The goal of using an SQL-backed dictionary is to enable some
  * other process (machine-learning algo) to dynamically update
@@ -41,75 +41,100 @@
  * what can be found in the file-backed dictionary.
  *
  * This does NOT support braces {} used to indicate optional connectors,
- * nor does it support the multi-connector at-sign @ and so using these
- * in the SQL tables is NOT valid! It also does not support cost
- * brackets [] and it does no support 'or'. This is really really
- * really supposed to be one unique disjunct per SQL table row!
- * This is NOT meant to be "just like the text files, but different".
+ * so using this in the SQL tables is NOT valid! It also does not
+ * support cost brackets []. Costs are handled out-of-line.
+ *
+ * This is not meant to be "just like the text files, but different".
  */
 
-static Exp * make_expression(Dictionary dict, const char *exp_str)
+static const char * make_expression(Dictionary dict,
+                                    const char *exp_str, Exp** pex)
 {
-	Exp* e;
-	Exp* and;
-	Exp* rest;
-	E_list *ell, *elr;
-
-	char *constr = NULL;
+	*pex = NULL;
+	/* Search for the start of a connector */
+	Exp_type etype = CONNECTOR_type;
 	const char * p = exp_str;
-	const char * con_start = NULL;
+	while (*p && (lg_isspace(*p))) p++;
+	if (0 == *p) return p;
 
-	/* search for the start of a connector */
-	while (*p && (lg_isspace(*p) || '&' == *p)) p++;
-	con_start = p;
-
-	if (0 == *p) return NULL;
-
-	/* search for the end of a connector */
-	while (*p && (isalnum(*p) || '*' == *p)) p++;
-
-	if (0 == *p) return NULL;
-
-	/* Connectors always end with a + or - */
-	assert (('+' == *p) || ('-' == *p),
-			"Missing direction character in connector string: %s", con_start);
-
-	/* Create an expression to hold the connector */
-	e = malloc(sizeof(Exp));
-	e->dir = *p;
-	e->type = CONNECTOR_type;
-	e->cost = 0.0;
-	if ('@' == *con_start)
+	/* If it's an open paren, assume its the begining of a new list */
+	if ('(' == *p)
 	{
-		constr = strndup(con_start+1, p-con_start-1);
-		e->multi = true;
+		p = make_expression(dict, ++p, pex);
 	}
 	else
 	{
-		constr = strndup(con_start, p-con_start);
-		e->multi = false;
+		/* Search for the end of a connector */
+		const char * con_start = p;
+		while (*p && (isalnum(*p) || '*' == *p)) p++;
+
+		/* Connectors always end with a + or - */
+		assert (('+' == *p) || ('-' == *p),
+				"Missing direction character in connector string: %s", con_start);
+
+		/* Create an expression to hold the connector */
+		Exp* e = malloc(sizeof(Exp));
+		e->dir = *p;
+		e->type = CONNECTOR_type;
+		e->cost = 0.0;
+		e->u.l = NULL;
+		char * constr = NULL;
+		if ('@' == *con_start)
+		{
+			constr = strndup(con_start+1, p-con_start-1);
+			e->multi = true;
+		}
+		else
+		{
+			constr = strndup(con_start, p-con_start);
+			e->multi = false;
+		}
+
+		e->u.condesc = condesc_add(&dict->contable,
+		                           string_set_add(constr, dict->string_set));
+		free(constr);
+		*pex = e;
 	}
 
-	e->u.condesc = condesc_add(&dict->contable,
-	                           string_set_add(constr, dict->string_set));
-	free(constr);
+	/* Is there any more? If not, return what we've got. */
+	p++;
+	while (*p && (lg_isspace(*p))) p++;
+	if (')' == *p || 0 == *p)
+	{
+		return p;
+	}
 
-	rest = make_expression(dict, ++p);
-	if (NULL == rest)
-		return e;
+	/* Wait .. there's more! */
+	if ('&' == *p)
+	{
+		etype = AND_type; p++;
+	}
+	else if ('o' == *p && 'r' == *(p+1))
+	{
+		etype = OR_type; p+=2;
+	}
+	else
+	{
+		assert(false, "Bad rest of expression %s", exp_str);
+	}
 
-	/* Join it all together with an AND node */
-	and = malloc(sizeof(Exp));
-	and->type = AND_type;
-	and->cost = 0.0;
-	and->u.l = ell = malloc(sizeof(E_list));
-	ell->next = elr = malloc(sizeof(E_list));
+	Exp* rest = NULL;
+	p = make_expression(dict, p, &rest);
+	assert(NULL != rest, "Badly formed expression %s", exp_str);
+
+	/* Join it all together. */
+	Exp* join = malloc(sizeof(Exp));
+	join->type = etype;
+	join->cost = 0.0;
+	E_list *ell = join->u.l = malloc(sizeof(E_list));
+	E_list *elr = ell->next = malloc(sizeof(E_list));
 	elr->next = NULL;
 
-	ell->e = e;
+	ell->e = *pex;
 	elr->e = rest;
+	*pex = join;
 
-	return and;
+	return p;
 }
 
 
@@ -146,34 +171,88 @@ static int exp_cb(void *user_data, int argc, char **argv, char **colName)
 	assert(2 == argc, "Bad column count");
 	assert(argv[0], "NULL column value");
 
-	bs->exp = make_expression(bs->dict, argv[0]);
+	Exp* exp = NULL;
+	make_expression(bs->dict, argv[0], &exp);
+	assert(NULL != exp, "Failed expression %s", argv[0]);
 
-	if (bs->exp)
-		if (!strtodC(argv[1], &bs->exp->cost))
-		{
-			prt_error("Warning: Invalid cost \"%s\" in expression \"%s\" "
-			          "(using 1.0)\n", argv[1], argv[0]);
-			bs->exp->cost = 1.0;
-		}
+	if (!strtodC(argv[1], &exp->cost))
+	{
+		prt_error("Warning: Invalid cost \"%s\" in expression \"%s\" "
+		          "(using 1.0)\n", argv[1], argv[0]);
+		exp->cost = 1.0;
+	}
+
+	if (NULL == bs->exp)
+	{
+		bs->exp = exp;
+		return 0;
+	}
+
+	E_list *ell, *elr;
+	Exp* orn = malloc(sizeof(Exp));
+	orn->type = OR_type;
+	orn->cost = 0.0;
+	orn->u.l = ell = malloc(sizeof(E_list));
+	ell->next = elr = malloc(sizeof(E_list));
+	elr->next = NULL;
+
+	ell->e = bs->exp;
+	elr->e = exp;
+	bs->exp = orn;
 
 	return 0;
 }
 
+/* Escape single-quotes.  That is, replace single-quotes by
+ * two single-quotes. e.g. don't --> don''t */
+static char * escape_quotes(const char * s)
+{
+	char * q = strchr(s, '\'');
+	if (NULL == q) return (char *) s;
+
+	/* If there are two in a row, already, assume that they are
+	 * already escaped. This is pathological, and a sign that
+	 * something somewhere else has already gone wrong. But I'm
+	 * exhausted debugging right now and will punt on this.
+	 */
+	if ('\'' == *(q+1)) return (char *) s;
+
+	char * es = malloc(2 * strlen(s) + 1);
+	char * p = es;
+	while (q)
+	{
+		strncpy(p, s, q-s+1);
+		p += q-s+1;
+		*p = '\'';
+		p++;
+		s = q+1;
+		q = strchr(s, '\'');
+	}
+	strcpy(p, s);
+	return es;
+}
+
 static void
 db_lookup_exp(Dictionary dict, const char *s, cbdata* bs)
 {
 	sqlite3 *db = dict->db_handle;
 	dyn_str *qry;
 
+	/* Escape single-quotes.  That is, replace single-quotes by
+	 * two single-quotes. e.g. don't --> don''t */
+	char * es = escape_quotes(s);
+
 	/* The token to look up is called the 'morpheme'. */
 	qry = dyn_str_new();
 	dyn_strcat(qry, "SELECT disjunct, cost FROM Disjuncts WHERE classname = \'");
-	dyn_strcat(qry, s);
+	dyn_strcat(qry, es);
 	dyn_strcat(qry, "\';");
 
 	sqlite3_exec(db, qry->str, exp_cb, bs, NULL);
 	dyn_str_delete(qry);
 
+	if (es != s) free(es);
+
 	if (4 < verbosity)
 	{
 		printf("Found expression for class %s: ", s);
@@ -197,29 +276,26 @@ static int exists_cb(void *user_data, int argc, char **argv, char **colName)
 /* callback -- set bs->dn to the dict nodes for a word in the dict */
 static int morph_cb(void *user_data, int argc, char **argv, char **colName)
 {
-	cbdata* bs = user_data;
-	Dict_node *dn;
-	char *scriword, *wclass;
-
 	assert(2 == argc, "Bad column count");
 	assert(argv[0], "NULL column value");
-	scriword = argv[0];
-	wclass = argv[1];
+	char * scriword = argv[0];
+	char * wclass = argv[1];
 
 	/* Now look up the expressions for each word */
+	cbdata* bs = user_data;
 	bs->exp = NULL;
 	db_lookup_exp(bs->dict, wclass, bs);
 
 	/* Well, if we found a classname for a word, then there really,
 	 * really should be able to find one or more corresponding disjuncts.
 	 * However, it is possible to have corrupted databases which do not
-	 * have any disjuncts for a word class.   We silently ignore these.
-	 * Although maybe we should throw an error here?
+	 * have any disjuncts for a word class.  We complain about those.
 	 */
-	if (NULL == bs->exp) return 0;
+	assert(NULL != bs->exp, "Missing disjuncts for word %s %s",
+		scriword, wclass);
 
 	/* Put each word into a Dict_node. */
-	dn = malloc(sizeof(Dict_node));
+	Dict_node *dn = malloc(sizeof(Dict_node));
 	memset(dn, 0, sizeof(Dict_node));
 	dn->string = string_set_add(scriword, bs->dict->string_set);
 	dn->right = bs->dn;
@@ -237,14 +313,20 @@ db_lookup_common(Dictionary dict, const char *s, const char *equals,
 	sqlite3 *db = dict->db_handle;
 	dyn_str *qry;
 
+	/* Escape single-quotes.  That is, replace single-quotes by
+	 * two single-quotes. e.g. don't --> don''t */
+	char * es = escape_quotes(s);
+
 	/* The token to look up is called the 'morpheme'. */
 	qry = dyn_str_new();
 	dyn_strcat(qry, "SELECT subscript, classname FROM Morphemes WHERE morpheme ");
 	dyn_strcat(qry, equals);
 	dyn_strcat(qry, " \'");
-	dyn_strcat(qry, s);
+	dyn_strcat(qry, es);
 	dyn_strcat(qry, "\';");
 
+	if (es != s) free(es);
+
 	sqlite3_exec(db, qry->str, cb, bs, NULL);
 	dyn_str_delete(qry);
 }
