@@ -364,12 +364,8 @@ Disjunct *eliminate_duplicate_disjuncts(Disjunct *dw)
 			if (d->dup_hash != dx->dup_hash) continue;
 			if (disjuncts_equal(dx, d)) break;
 		}
-		if (dx == NULL)
-		{
-			d->dup_table_next = dt->dup_table[h];
-			dt->dup_table[h] = d;
-		}
-		else
+
+		if (dx != NULL)
 		{
 			/* Discard the current disjunct. */
 			if (d->cost < dx->cost) dx->cost = d->cost;
@@ -379,9 +375,13 @@ Disjunct *eliminate_duplicate_disjuncts(Disjunct *dw)
 
 			count++;
 			*dd = d->next; /* NEXT - set current disjunct to the next one. */
-			continue;
 		}
-		dd = &d->next; /* NEXT */
+		else
+		{
+			d->dup_table_next = dt->dup_table[h];
+			dt->dup_table[h] = d;
+			dd = &d->next; /* NEXT */
+		}
 	}
 
 	lgdebug(+D_DISJ+(0==count)*1000, "Killed %u duplicates\n", count);
@@ -525,9 +525,8 @@ void print_all_disjuncts(Sentence sent)
 /* ============= Connector encoding, sharing and packing ============= */
 
 /*
- * sentence_pack() copies the disjuncts and connectors to a continuous
- * memory block. This facilitate a better memory caching for long
- * sentences.
+ * sentence_pack() copies the disjuncts and connectors to a contiguous
+ * memory. This facilitate a better memory caching for long sentences.
  *
  * In addition, it shares the memory of identical trailing connector
  * sequences, aka "tracons". Tracons are considered identical if they
@@ -617,8 +616,9 @@ void print_all_disjuncts(Sentence sent)
  * to disjuncts of different alternatives may have different linkage
  * counts because some alternatives-connectivity checks (to the middle
  * disjunct) are done in the fast-matcher. These restrictions are
- * implemented by using different tracon IDs per Gword.
- *
+ * implemented by using a different tracon ID per Gword (FIXME - this is
+ * more strict then needed - a different tracon IDs per alternative would
+ * suffice).
  * The tracon memory sharing is currently not directly used in the
  * parsing algo besides reducing the needed CPU cache by a large factor.
  *
@@ -651,7 +651,7 @@ void print_all_disjuncts(Sentence sent)
  * here may need to be reflected in the comments there too.
  */
 
-static void id_table_check(Tracon_list *tl, unsigned int index, int dir)
+static void tlsz_check(Tracon_list *tl, unsigned int index, int dir)
 {
 
 	if (index >= tl->table_size[dir])
@@ -676,26 +676,26 @@ static Connector *pack_connectors(Tracon_sharing *ts, Connector *origc, int dir,
 	Connector head;
 	Connector *prevc = &head;
 	Connector *newc = &head;
-	Connector *lcblock = ts->cblock; /* For convenience. */
-	Tracon_list *tl = ts->tracon_list;
+	Connector *lcblock = ts->cblock;     /* For convenience. */
+	Tracon_list *tl = ts->tracon_list;   /* If non-NULL - encode for pruning. */
 
 	for (Connector *o = origc; NULL != o;  o = o->next)
 	{
 		newc = NULL;
 
-		if (NULL != ts)
+		if ((NULL != ts) && (NULL != ts->csid[dir]))
 		{
-			/* Encoding is used - share trailing connector sequences. */
+			/* Encoding is used - share tracons. */
 			Connector **tracon = tracon_set_add(o, ts->csid[dir]);
 
 			if (NULL == *tracon)
 			{
-				/* The first time we encounter this connector sequence. */
+				/* The first time we encounter this tracon. */
 				*tracon = lcblock; /* Save its future location in the tracon_set. */
 
-				if (NULL != tl)
+				if (ts->is_pruning)
 				{
-					id_table_check(tl, tl->entries[dir], dir);
+					tlsz_check(tl, tl->entries[dir], dir);
 					uint32_t cblock_index = (uint32_t)(lcblock - ts->cblock_base);
 					tl->table[dir][tl->entries[dir]] = cblock_index;
 					tl->entries[dir]++;
@@ -704,15 +704,15 @@ static Connector *pack_connectors(Tracon_sharing *ts, Connector *origc, int dir,
 			else
 			{
 				newc = *tracon;
-				if (NULL == tl)
+				if (!ts->is_pruning)
 				{
 					if (o->nearest_word != newc->nearest_word)
 					{
 						/* This is a rare case in which a shallow and deep
 						 * connectors don't have the same nearest_word, because
 						 * a shallow connector may mach a deep connector
-						 * earlier. Because the nearest word is different, we we
-						 * cannot share it. (Such shallow and deep Tracons could
+						 * earlier. Because the nearest word is different, we
+						 * cannot share it. (Such shallow and deep tracons could
 						 * be shared separately, but because this is a rare
 						 * event there is no need to do that.)
 						 * Note:
@@ -727,32 +727,37 @@ static Connector *pack_connectors(Tracon_sharing *ts, Connector *origc, int dir,
 
 		if (newc == NULL)
 		{
-			/* No sharing is done. */
+			/* No sharing yet. */
 			newc = lcblock++;
 			*newc = *o;
 
-			if (NULL == ts->tracon_list)
+			if (ts->is_pruning)
 			{
-				/* For the parsing sharing we need a unique ID. */
-				newc->tracon_id = ts->next_id[dir]++;
+				/* Initializations for the pruning step. */
+				newc->refcount = 1;  /* The first connector at this location. */
+				newc->tracon_id = 0; /* Used in power_prune() for pass number. */
+				if (NULL != tl)
+					tl->num_cnctrs_per_word[dir][w]++;
 			}
 			else
 			{
-				newc->refcount = 1; /* No sharing yet. */
-				newc->tracon_id = 0;
-				tl->num_cnctrs_per_word[dir][w]++;
+				/* For the parsing step we need a unique ID. */
+				newc->tracon_id = ts->next_id[dir]++;
+#ifdef DEBUG
+				newc->refcount = 0;  /* Not used; zero for debug consistency. */
+#endif
 			}
 		}
 		else
 		{
-			if (NULL != ts->tracon_list)
+			if (ts->is_pruning)
 			{
 				for (Connector *n = newc; NULL != n; n = n->next)
 					n->refcount++;
 			}
 			prevc->next = newc;
 
-			/* Just shared a trailing connector sequence, nothing more to do. */
+			/* Just shared a tracon, nothing more to do. */
 			ts->cblock = lcblock;
 			return head.next;
 		}
@@ -766,36 +771,33 @@ static Connector *pack_connectors(Tracon_sharing *ts, Connector *origc, int dir,
 	return head.next;
 }
 
-#define WORD_OFFSET 256 /* Reserved for null connectors. */
-
 /**
- *  Set dummy tracon_id's.
- *  To be used for short sentences.
+ * Set dummy tracon_id's.
+ * To be used for short sentences (for which a full encoding is too
+ * costly) or for library tests that actually bypass the use of tracon IDs
+ * (to validate that the tracon_id implementation didn't introduce bugs).
  */
-static int enumerate_connectors_sequentially(Sentence sent)
+static int enumerate_connectors_sequentially(Tracon_sharing *ts)
 {
-	int id = WORD_OFFSET;
+	int id = ts->word_offset;
 
-	for (WordIdx w = 0; w < sent->length; w++)
+	for (size_t i = 0; i < ts->num_disjuncts; i++)
 	{
-		for (Disjunct *d = sent->word[w].d; NULL != d; d = d->next)
-		{
-			for (Connector *c = d->left; NULL != c; c = c->next)
-			{
-				c->tracon_id = id++;
-			}
-			for (Connector *c = d->right; NULL != c; c = c->next)
-			{
-				c->tracon_id = id++;
-			}
-		}
+		Disjunct *d = &ts->dblock_base[i];
+
+		for (Connector *c = d->left; NULL != c; c = c->next)
+			c->tracon_id = id++;
+
+		for (Connector *c = d->right; NULL != c; c = c->next)
+			c->tracon_id = id++;
+
 	}
 
 	return id + 1;
 }
 
 /**
- * Pack the given disjunct chain in a continuous memory block.
+ * Pack the given disjunct chain in a contiguous memory block.
  * If the disjunct is NULL, return NULL.
  */
 static Disjunct *pack_disjuncts(Tracon_sharing *ts, Disjunct *origd, int w)
@@ -813,15 +815,18 @@ static Disjunct *pack_disjuncts(Tracon_sharing *ts, Disjunct *origd, int w)
 		newd->cost = t->cost;
 		newd->originating_gword = t->originating_gword;
 
-		if (NULL == ts->tracon_list)
+		if (!ts->is_pruning)
 		    token = (uintptr_t)t->originating_gword;
 
 		if (token != ts->last_token)
 		{
 			ts->last_token = token;
-			tracon_set_reset(ts->csid[0]);
-			tracon_set_reset(ts->csid[1]);
-			//printf("TOKEN token %ld\n", token);
+			if (NULL != ts->csid[0])
+			{
+				//printf("Token %ld\n", token);
+				tracon_set_reset(ts->csid[0]);
+				tracon_set_reset(ts->csid[1]);
+			}
 		}
 		newd->left = pack_connectors(ts, t->left, 0, w);
 		newd->right = pack_connectors(ts, t->right, 1,  w);
@@ -835,34 +840,39 @@ static Disjunct *pack_disjuncts(Tracon_sharing *ts, Disjunct *origd, int w)
 	return head.next;
 }
 
-#define ID_TABLE_SZ 8192 /* Initial size of the tracon_id table */
+#define TLSZ 8192         /* Initial size of the tracon list table */
+
+/* Reserved tracon ID space for NULL connectors (zero-length tracons).
+ * Currently, tracons are unique per word. So this is actually the max.
+ * number of words in a sentence rounded up to a power of 2.
+ * FIXME: Derive it from MAX_SENTENCE. */
+#define WORD_OFFSET 256
 
 /** Create a context descriptor for disjuncts & connector memory "packing".
- * - Allocate a memory block for all the disjuncts & connectors.
- *   The current Connector struct size is 32 bit, and the intention is
+ *   Allocate a memory block for all the disjuncts & connectors.
+ *   The current Connector struct size is 32 bytes, and the intention is
  *   to keep it with a power-of-2 size. The idea is to put an integral
  *   number of connectors in each cache line (assumed to be >= Connector
  *   struct size, e.g. 64 bytes), so one connector will not need 2 cache
  *   lines.
  *
- *   The allocated memory block includes 3 sections , in that order:
- *   1. A block for disjuncts, when it start is not aligned (the
- *   disjunct size is currently 56 bytes and cannot be reduced much).
- *   2. A small alignment gap, that ends in a 64-byte boundary.
- *   3. A block of connectors, which is so aligned to 64-byte boundary.
+ *   The current Disjunct struct size is 64 bytes, and the intention is
+ *   to keep it at this size for performance reasons.
  *
- *   NOTE: Recently the disjunct size got increases to 64 bytes and
- *   the intention is to keep it at this size. So the alignment code
- *   that implements the above is now a kind of no-op.
+ *   The allocated memory block includes 2 sections, in that order:
+ *   1. A block for disjuncts.
+ *   2. A block of connectors.
  *
- * - If the packing is done for the pruning step, allocate Tracon list
- *   stuff too. In that case also call tracon_set_shallow() so Tracons
+ *   If encoding is done for the pruning step, allocate tracon list
+ *   stuff too. In that case also call tracon_set_shallow() so tracons
  *   starting with a shallow connector will be considered different than
  *   similar ones starting with a deep connector.
  *
+ * @param is_pruning TRUE if invoked for pruning, FALSE if invoked for parsing.
  * @return The said context descriptor.
  */
-static Tracon_sharing *pack_sentence_init(Sentence sent, bool is_pruning)
+static Tracon_sharing *pack_sentence_init(Sentence sent, bool is_pruning,
+                                          bool do_encoding)
 {
 	unsigned int dcnt = 0;
 	unsigned int ccnt = 0;
@@ -874,7 +884,8 @@ static Tracon_sharing *pack_sentence_init(Sentence sent, bool is_pruning)
 	size_t dsize = dcnt * sizeof(Disjunct);
 	dsize = ALIGN(dsize, CONN_ALIGNMENT); /* Align connector block. */
 	size_t csize = ccnt * sizeof(Connector);
-	void *memblock = malloc(dsize + csize);
+	size_t memblock_sz = dsize + csize;
+	void *memblock = malloc(memblock_sz);
 	Disjunct *dblock = memblock;
 	Connector *cblock = (Connector *)((char *)memblock + dsize);
 
@@ -882,6 +893,7 @@ static Tracon_sharing *pack_sentence_init(Sentence sent, bool is_pruning)
 	memset(ts, 0, sizeof(Tracon_sharing));
 
 	ts->memblock = memblock;
+	ts->memblock_sz = memblock_sz;
 	ts->cblock_base = cblock;
 	ts->cblock = cblock;
 	ts->dblock = dblock;
@@ -890,23 +902,26 @@ static Tracon_sharing *pack_sentence_init(Sentence sent, bool is_pruning)
 	ts->word_offset = is_pruning ? 1 : WORD_OFFSET;
 	ts->next_id[0] = ts->next_id[1] = ts->word_offset;
 	ts->last_token = (uintptr_t)-1;
+	ts->is_pruning = is_pruning;
 
-	ts->csid[0] = tracon_set_create();
-	ts->csid[1] = tracon_set_create();
-
-	if (is_pruning)
+	if (do_encoding)
 	{
-		ts->tracon_list = malloc(sizeof(Tracon_list));
-		memset(ts->tracon_list, 0, sizeof(Tracon_list));
-		ts->tracon_list->memblock_sz = dsize + csize;
-		unsigned int **ncpw = ts->tracon_list->num_cnctrs_per_word;
-		for (int dir = 0; dir < 2; dir++)
-		{
-			ncpw[dir] = malloc(sent->length * sizeof(**ncpw));
-			memset(ncpw[dir], 0, sent->length * sizeof(**ncpw));
+		ts->csid[0] = tracon_set_create();
+		ts->csid[1] = tracon_set_create();
 
-			tracon_set_shallow(true, ts->csid[dir]);
-			id_table_check(ts->tracon_list, ID_TABLE_SZ, dir); /* Allocate table. */
+		if (is_pruning)
+		{
+			ts->tracon_list = malloc(sizeof(Tracon_list));
+			memset(ts->tracon_list, 0, sizeof(Tracon_list));
+			unsigned int **ncpw = ts->tracon_list->num_cnctrs_per_word;
+			for (int dir = 0; dir < 2; dir++)
+			{
+				ncpw[dir] = malloc(sent->length * sizeof(**ncpw));
+				memset(ncpw[dir], 0, sent->length * sizeof(**ncpw));
+
+				tracon_set_shallow(true, ts->csid[dir]);
+				tlsz_check(ts->tracon_list, TLSZ, dir); /* Allocate table. */
+			}
 		}
 	}
 
@@ -924,57 +939,67 @@ void free_tracon_sharing(Tracon_sharing *ts)
 			free(ts->tracon_list->num_cnctrs_per_word[dir]);
 			free(ts->tracon_list->table[dir]);
 		}
-		tracon_set_delete(ts->csid[dir]);
-		ts->csid[dir] = NULL;
+		if (NULL != ts->csid[dir])
+		{
+			tracon_set_delete(ts->csid[dir]);
+			ts->csid[dir] = NULL;
+		}
 	}
-	free(ts->tracon_list);
-	ts->tracon_list = NULL;
 
+	if (NULL != ts->tracon_list)
+	{
+		free(ts->tracon_list);
+		ts->tracon_list = NULL;
+	}
 	free(ts);
 }
 
 /**
- * Pack all disjunct and connectors into a one big memory block.
- * This facilitate a better memory caching for long sentences
- * (a performance gain of a few percents).
+ * Pack all disjunct and connectors into a one big memory block, share
+ * tracon memory and generate tracon IDs (for parsing) or tracon lists
+ * with reference count (for pruning). Aka "connector encoding".
  *
- * The current Connector struct size is 32 bit, and future ones may be
- * smaller, but still with a power-of-2 size.
- * The idea is to put an integral number of connectors in each cache line
- * (assumed to be >= Connector struct size, e.g. 64 bytes),
- * so one connector will not need 2 cache lines.
+ * The disjunct and connectors packing in a contiguous memory facilitate a
+ * better memory caching for long sentences (a performance gain of a few
+ * percents).
  *
- * The allocated memory includes 3 sections , in that order:
- * 1. A block for disjuncts, when it start is not aligned (the disjunct size
- * is currently 56 bytes and cannot be reduced much).
- * 2. A small alignment gap, that ends in a 64-byte boundary.
- * 3. A block of connectors, which is so aligned to 64-byte boundary.
+ * The tracon IDs (if invoked for the parsing step) or tracon lists (if
+ * invoked for pruning step) allow for a huge performance boost at these
+ * steps.
  *
- * A trailing sequence encoding and sharing is done too.
- * Note: Connector sharing, trailing hash and packing always go together.
+ * Note:
+ * In order to save overhead, sentences shorter than
+ * sent->min_len_encoding don't undergo encoding - only packing.
+ * This can also be used for library tests that totally bypass the use of
+ * connector encoding (to validate that the
+ * tracon_id/packing/sharing/refcount implementation didn't introduce bugs
+ * in the pruning and parsing steps).
+ * E.g. when using link-parser:
+ * - To entirely disable connector encoding:
+ * link-parser -test=len-trailing-hash:254
+ * - To use connector encoding even for short sentences:
+ * link-parser -test=len-trailing-hash:0
+ * Any different result (e.g. number of discarded disjuncts in the pruning
+ * step or different parsing results) indicates a bug.
  */
 static Tracon_sharing *pack_sentence(Sentence sent, bool is_pruning)
 {
-	bool do_share = sent->length >= sent->min_len_encoding;
+	bool do_encoding = sent->length >= sent->min_len_encoding;
 	Tracon_sharing *ts;
 
-	if (!do_share)
-	{
-		if (!is_pruning)
-		{
-			lgdebug(D_DISJ, "enumerate_connectors_sequentially\n");
-			enumerate_connectors_sequentially(sent);
-		}
-		return NULL;
-	}
-
-	ts = pack_sentence_init(sent, is_pruning);
+	ts = pack_sentence_init(sent, is_pruning, do_encoding);
 
 	for (WordIdx w = 0; w < sent->length; w++)
 	{
 		sent->word[w].d = pack_disjuncts(ts, sent->word[w].d, w);
 	}
 
+	if (is_pruning && !do_encoding)
+	{
+		lgdebug(D_DISJ, "enumerate_connectors_sequentially\n");
+		enumerate_connectors_sequentially(ts);
+	}
+
 	/* On long sentences, many MB of connector-space are saved, but we
 	 * cannot use a realloc() here without the overhead of relocating
 	 * the pointers in the used part of memblock (if realloc() returns a
@@ -987,9 +1012,9 @@ Tracon_sharing *pack_sentence_for_pruning(Sentence sent)
 {
 	Tracon_sharing *ts = pack_sentence(sent, true);
 
-	if (ts == NULL)
+	if (NULL == ts->csid[0])
 	{
-		lgdebug(D_DISJ, "Debug: Encode for pruning (len %zu): None",
+		lgdebug(D_DISJ, "Debug: Encode for pruning (len %zu): None\n",
 		        sent->length);
 	}
 	else
@@ -1009,9 +1034,9 @@ Tracon_sharing *pack_sentence_for_parsing(Sentence sent)
 {
 	Tracon_sharing *ts = pack_sentence(sent, false);
 
-	if (ts == NULL)
+	if (NULL == ts->csid[0])
 	{
-		lgdebug(D_DISJ, "Debug: Encode for parsing (len %zu): None",
+		lgdebug(D_DISJ, "Debug: Encode for parsing (len %zu): None\n",
 		        sent->length);
 	}
 	else
@@ -1030,13 +1055,11 @@ Tracon_sharing *pack_sentence_for_parsing(Sentence sent)
 /* ============ Save and restore sentence disjuncts ============ */
 void *save_disjuncts(Sentence sent, Tracon_sharing *ts, Disjunct **disjuncts)
 {
-	Tracon_list *tl = ts->tracon_list;
-
 	for (WordIdx w = 0; w < sent->length; w++)
 		disjuncts[w] = sent->word[w].d;
 
-	void *saved_memblock = malloc(tl->memblock_sz);
-	memcpy(saved_memblock, ts->memblock, ts->tracon_list->memblock_sz);
+	void *saved_memblock = malloc(ts->memblock_sz);
+	memcpy(saved_memblock, ts->memblock, ts->memblock_sz);
 
 	return saved_memblock;
 }
@@ -1049,5 +1072,5 @@ void restore_disjuncts(Sentence sent, Disjunct **disjuncts,
 	for (WordIdx w = 0; w < sent->length; w++)
 		sent->word[w].d = disjuncts[w];
 
-	memcpy(ts->memblock, saved_memblock, ts->tracon_list->memblock_sz);
+	memcpy(ts->memblock, saved_memblock, ts->memblock_sz);
 }
