@@ -4,7 +4,7 @@
 /* All rights reserved                                                   */
 /*                                                                       */
 /* Use of the link grammar parsing system is subject to the terms of the */
-/* license set forth in the LICENSE file included with this softwares.   */
+/* license set forth in the LICENSE file included with this software.   */
 /* This license allows free redistribution and use in source and binary  */
 /* forms, with or without modification, subject to certain conditions.   */
 /*                                                                       */
@@ -71,9 +71,13 @@ typedef struct
 } Disjuncts_desc_t;
 
 /* Trailing connector sequences (aka tracons) are memory-shared for the
- * benefit of the pruning and parsing stages. Basically each unique
- * tracon gets its own tracon_id.
- * For the parsing stage, on each pass the tracon_id of "good"
+ * benefit of the pruning and parsing stages. To that end, unique tracons
+ * are identified. For the parsing stage, a unique tracon_id is assigned
+ * to the Connector's tracon_id field. For the pruning stage, this field
+ * remains 0 (later used for pruning pass_count - see below) and instead
+ * the Connector's refcount filed reflects the number of connectors which
+ * are memory-shared.
+ * In the pruning stage, on each pass the tracon_id of "good"
  * connectors is assigned the pass number so they will not be checked
  * again on the same pass. The pruning stage also uses the connector
  * refcount field as a reference count - the number of times this
@@ -97,25 +101,30 @@ typedef struct
 	/* Table of tracons. A 32bit index into the connector array in
 	 * memblock (instead of (Connector*)) is used for better use of the
 	 * CPU cache on 64-bit CPUs. */
-	uint32_t *table[2];         /* Indexed by tracon_id */
+	uint32_t *table[2];         /* Indexed by entry number */
 	size_t entries[2];          /* Actual number of entries */
 	size_t table_size[2];       /* Allocated number of entries */
-	size_t memblock_sz;         /* Pruning memblock size  */
 } Tracon_list;
 
 struct tracon_sharing_s
 {
-	void *memblock;             /* Memory block for disjuncts & connectors */
+	union
+	{
+		void *memblock;             /* Memory block for disjuncts & connectors */
+		Disjunct *dblock_base;      /* Start of disjunct block */
+	};
+	size_t memblock_sz;         /* memblock size */
 	Connector *cblock_base;     /* Start of connector block */
 	Connector *cblock;          /* Next available memory for connector */
-	unsigned int num_connectors;
 	Disjunct *dblock;           /* Next available memory for disjunct */
+	unsigned int num_connectors;
 	unsigned int num_disjuncts;
+	Tracon_set *csid[2];        /* For generating unique tracon IDs */
+	int next_id[2];             /* Next unique tracon ID */
+	uintptr_t last_token;       /* Tracons are only unique per "token" */
 	int word_offset;            /* Start number for connector tracon_id */
-	Tracon_set *csid[2];        /* For generating unique IDs */
-	int next_id[2];
-	uintptr_t last_token;
-	Tracon_list *tracon_list;
+	bool is_pruning;            /* true - for pruning; false - for parsing */
+	Tracon_list *tracon_list;   /* Used only for pruning */
 };
 
 void *save_disjuncts(Sentence, Tracon_sharing *, Disjunct **);
