@@ -46,6 +46,9 @@ link_public_api(const char *)
 link_public_api(const char *)
 	linkgrammar_get_dict_version(Dictionary);
 
+link_public_api(const char *)
+	linkgrammar_get_configuration(void);
+
 link_public_api(const char *)
 	linkgrammar_get_dict_locale(Dictionary);
 
@@ -87,6 +90,8 @@ link_public_api(int)
      lg_error_printall(lg_error_handler, void *data);
 link_public_api(int)
      lg_error_clearall(void);
+link_public_api(bool)
+     lg_error_flush(void);
 
 /**********************************************************************
  *
@@ -108,6 +113,8 @@ link_public_api(void)
      dictionary_set_data_dir(const char * path);
 link_public_api(char *)
      dictionary_get_data_dir(void);
+link_public_api(FILE *)
+	  linkgrammar_open_data_file(const char *);
 
 /**********************************************************************
  *
@@ -187,10 +194,6 @@ link_public_api(void)
      parse_options_set_use_sat_parser(Parse_Options opts, bool use_sat_solver);
 link_public_api(bool)
      parse_options_get_use_sat_parser(Parse_Options opts);
-link_public_api(void)
-     parse_options_set_use_viterbi(Parse_Options opts, bool use_viterbi);
-link_public_api(bool)
-     parse_options_get_use_viterbi(Parse_Options opts);
 link_public_api(bool)
      parse_options_timer_expired(Parse_Options opts);
 link_public_api(bool)
@@ -271,6 +274,8 @@ link_public_api(double)
      sentence_disjunct_cost(Sentence sent, LinkageIdx linkage_num);
 link_public_api(int)
      sentence_link_cost(Sentence sent, LinkageIdx linkage_num);
+link_public_api(bool)
+     sentence_display_wordgraph(Sentence sent, const char *modestr);
 
 /**********************************************************************
  *
@@ -359,22 +364,32 @@ link_public_api(double)
 link_public_api(const char *)
      linkage_get_violation_name(const Linkage linkage);
 
+/* Experimental API - subject to changes. */
+link_public_api(WordIdx)
+     linkage_get_word_byte_start(const Linkage linkage, WordIdx w);
+link_public_api(WordIdx)
+     linkage_get_word_byte_end(const Linkage linkage, WordIdx w);
+link_public_api(WordIdx)
+     linkage_get_word_char_start(const Linkage linkage, WordIdx w);
+link_public_api(WordIdx)
+     linkage_get_word_char_end(const Linkage linkage, WordIdx w);
+
 
 /**********************************************************************
  *
  * Internal functions -- do not use these in new code!
- * These are not intended for general public use, but are required to
- * get the link-parser executable to link under MSVC6.
- * XXX FIXME we're on msvc14 now, do we still need these??
+ * They are used in the link-parser code (which needs their signature to
+ * compile), but are not intended for general public use.
  *
  ***********************************************************************/
 
-link_public_api(void)
+link_public_api(size_t)
+     utf8_strwidth(const char *);
+
+link_public_api(char *)
      dict_display_word_expr(Dictionary dict, const char *, Parse_Options opts);
-link_public_api(void)
+link_public_api(char *)
      dict_display_word_info(Dictionary dict, const char *, Parse_Options opts);
-link_public_api(void)
-     left_print_string(FILE* fp, const char *, int);
 link_public_api(bool)
      lg_expand_disjunct_list(Sentence sent);
 
@@ -382,7 +397,7 @@ link_public_api(bool)
  *
  * Internal functions -- do not use these in new code!
  * These are not intended for general public use, but are required to
- * work around certain Micorsoft Windows linking oddities
+ * work around certain Microsoft Windows linking oddities
  * (specifically, to be callable from the JNI bindings library.)
  *
  ***********************************************************************/
@@ -413,29 +428,13 @@ link_public_api(int)
 #define GNUC_DEPRECATED
 #endif
 
-#if defined(_MSC_VER) && _MSC_VER > 1200  /* Only if newer than MSVC6 */ 
+#if defined(_MSC_VER)
  #define MS_DEPRECATED __declspec(deprecated)
 #else
  #define MS_DEPRECATED
 #endif
 
-/**********************************************************************
- *
- * Functions that allow special-purpose post-processing of linkages
- *
- ***********************************************************************/
-
 #ifndef SWIG
-/** Do not use in new code! */
-typedef struct Postprocessor_s PostProcessor;
-
-MS_DEPRECATED link_public_api(PostProcessor *)
-     post_process_open(const char *path) GNUC_DEPRECATED;
-MS_DEPRECATED link_public_api(void)
-     post_process_close(PostProcessor *) GNUC_DEPRECATED;
-MS_DEPRECATED link_public_api(void)
-     linkage_post_process(Linkage, PostProcessor *) GNUC_DEPRECATED;
-
 LINK_END_DECLS
 #endif /* !SWIG */
 
