@@ -241,45 +241,19 @@ void free_extractor(extractor_t * pex)
 	xfree((void *) pex, sizeof(extractor_t));
 }
 
-static inline unsigned int el_pair_hash(extractor_t * pex,
-                            int lw, int rw,
-                            const Connector *le, const Connector *re,
-                            unsigned int null_count)
-{
-	unsigned int i;
-	size_t table_size = pex->x_table_size;
-	int l_id = 0, r_id = 0;
-
-	if (NULL != le) l_id = le->suffix_id;
-	if (NULL != re) r_id = re->suffix_id;
-
-#ifdef DEBUG
-	assert(((NULL == le) || le->suffix_id) &&
-	       ((NULL == re) || re->suffix_id));
-#endif
-
-   /* sdbm-based hash */
-	i = null_count;
-	i = lw + (i << 6) + (i << 16) - i;
-	i = rw + (i << 6) + (i << 16) - i;
-	i = l_id + (i << 6) + (i << 16) - i;
-	i = r_id + (i << 6) + (i << 16) - i;
-
-	return i & (table_size-1);
-}
 /**
  * Returns the pointer to this info, NULL if not there.
- * Note that there is no need to use (lw, rw) as keys because suffix_id
- * is also encoded by the word number.
+ * Note that there is no need to use (lw, rw) as keys because tracon_id
+ * values are not shared between words.
  */
 static Pset_bucket * x_table_pointer(int lw, int rw,
                               Connector *le, Connector *re,
                               unsigned int null_count, extractor_t * pex)
 {
 	Pset_bucket *t;
-	t = pex->x_table[el_pair_hash(pex, lw, rw, le, re, null_count)];
-	int l_id = (NULL != le) ? le->suffix_id : lw;
-	int r_id = (NULL != re) ? re->suffix_id : rw;
+	int l_id = (NULL != le) ? le->tracon_id : lw;
+	int r_id = (NULL != re) ? re->tracon_id : rw;
+	t = pex->x_table[pair_hash(pex->x_table_size, lw, rw, l_id, r_id, null_count)];
 
 	for (; t != NULL; t = t->next) {
 		if ((t->set.l_id == l_id) && (t->set.r_id == r_id) &&
@@ -290,10 +264,6 @@ static Pset_bucket * x_table_pointer(int lw, int rw,
 
 /**
  * Stores the value in the x_table.  Assumes it's not already there.
- * Since disjuncts of connectors with the same suffix_id may have
- * different costs and may belong to different words, the disjunct cost
- * and word_string are saved, to be used as retrieval keys in
- * x_table_pointer().
  */
 static Pset_bucket * x_table_store(int lw, int rw,
                                   Connector *le, Connector *re,
@@ -306,13 +276,13 @@ static Pset_bucket * x_table_store(int lw, int rw,
 	n->set.lw = lw;
 	n->set.rw = rw;
 	n->set.null_count = null_count;
-	n->set.l_id = (NULL != le) ? le->suffix_id : lw;
-	n->set.r_id = (NULL != re) ? re->suffix_id : rw;
+	n->set.l_id = (NULL != le) ? le->tracon_id : lw;
+	n->set.r_id = (NULL != re) ? re->tracon_id : rw;
 	n->set.count = 0;
 	n->set.first = NULL;
 	n->set.tail = NULL;
 
-	h = el_pair_hash(pex, lw, rw, le, re, null_count);
+	h = pair_hash(pex->x_table_size, lw, rw, n->set.l_id, n->set.r_id, null_count);
 	t = pex->x_table[h];
 	n->next = t;
 	pex->x_table[h] = n;
@@ -418,7 +388,8 @@ Parse_set * mk_parse_set(fast_matcher_t *mchxt,
 		Parse_set* dummy;
 		Disjunct* dis;
 
-		if (!pex->islands_ok && (lw != -1)) return &xt->set;
+		if (!pex->islands_ok && (lw != -1) && (pex->words[lw].d != NULL))
+			return &xt->set;
 		if (null_count == 0) return &xt->set;
 
 		RECOUNT({xt->set.recount = 0;})
@@ -739,28 +710,38 @@ static void list_links(Linkage lkg, const Parse_set * set, int index)
 	 list_links(lkg, pc->set[1], index / pc->set[0]->count);
 }
 
-static void list_random_links(Linkage lkg, extractor_t * pex, const Parse_set * set)
+static void list_random_links(Linkage lkg, unsigned int *rand_state,
+                              const Parse_set * set)
 {
 	Parse_choice *pc;
 	int num_pc, new_index;
 
 	if (set == NULL || set->first == NULL) return;
-	num_pc = 0;
-	for (pc = set->first; pc != NULL; pc = pc->next) {
-		num_pc++;
+
+	/* Most of the times there is only one list element. */
+	if (set->first->next == NULL)
+	{
+		pc = set->first;
 	}
+	else
+	{
+		num_pc = 0;
+		for (pc = set->first; pc != NULL; pc = pc->next) {
+			num_pc++;
+		}
 
-	new_index = rand_r(&pex->rand_state) % num_pc;
+		new_index = rand_r(rand_state) % num_pc;
 
-	num_pc = 0;
-	for (pc = set->first; pc != NULL; pc = pc->next) {
-		if (new_index == num_pc) break;
-		num_pc++;
+		num_pc = 0;
+		for (pc = set->first; pc != NULL; pc = pc->next) {
+			if (new_index == num_pc) break;
+			num_pc++;
+		}
 	}
 
 	issue_links_for_choice(lkg, pc);
-	list_random_links(lkg, pex, pc->set[0]);
-	list_random_links(lkg, pex, pc->set[1]);
+	list_random_links(lkg, rand_state, pc->set[0]);
+	list_random_links(lkg, rand_state, pc->set[1]);
 }
 
 /**
@@ -776,7 +757,7 @@ void extract_links(extractor_t * pex, Linkage lkg)
 		bool repeatable = false;
 		if (0 == pex->rand_state) repeatable = true;
 		if (repeatable) pex->rand_state = index;
-		list_random_links(lkg, pex, pex->parse_set);
+		list_random_links(lkg, &pex->rand_state, pex->parse_set);
 		if (repeatable)
 			pex->rand_state = 0;
 		else
