@@ -97,7 +97,7 @@ void free_fast_matcher(Sentence sent, fast_matcher_t *mchxt)
 	}
 
 	free(mchxt->match_list);
-	lgdebug(6, "Sentence size %zu, match_list_size %zu\n",
+	lgdebug(6, "Sentence length %zu, match_list_size %zu\n",
 	        mchxt->size, mchxt->match_list_size);
 
 	xfree(mchxt->l_table_size, mchxt->size * sizeof(unsigned int));
@@ -169,15 +169,6 @@ static Match_node * add_to_left_table_list(Match_node * m, Match_node * l)
 	return l;  /* return pointer to original head */
 }
 
-/**
- * Compare only the uppercase part of two connectors.
- * Return true if they are the same, else false.
- */
-static bool con_uc_eq(const Connector *c1, const Connector *c2)
-{
-	return (connector_uc_num(c1) == connector_uc_num(c2));
-}
-
 static Match_node **get_match_table_entry(unsigned int size, Match_node **t,
                                           Connector * c, int dir)
 {
@@ -188,7 +179,7 @@ static Match_node **get_match_table_entry(unsigned int size, Match_node **t,
 	if (dir == 1) {
 		while (NULL != t[h])
 		{
-			if (con_uc_eq(t[h]->d->right, c)) break;
+			if (connector_uc_eq(t[h]->d->right, c)) break;
 
 			/* Increment and try again. Every hash bucket MUST have
 			 * a unique upper-case part, since later on, we only
@@ -204,7 +195,7 @@ static Match_node **get_match_table_entry(unsigned int size, Match_node **t,
 	{
 		while (NULL != t[h])
 		{
-			if (con_uc_eq(t[h]->d->left, c)) break;
+			if (connector_uc_eq(t[h]->d->left, c)) break;
 			h = (h + 1) & (size-1);
 			if (h == s) return NULL;
 		}
@@ -554,6 +545,8 @@ form_match_list(fast_matcher_t *ctxt, int w,
 #ifdef VERIFY_MATCH_LIST
 	static int id = 0;
 	int lid = ++id; /* A local copy, for multi-threading support. */
+#else
+	const int lid = 0;
 #endif
 
 	for (mx = mr; mx != NULL; mx = mx->next)
