@@ -258,7 +258,7 @@ static void sort_linkages(Sentence sent, Parse_Options opts)
  * is done by pack_sentence_for_pruning(), and it is not done for
  * sentences shorter than a certain limit. In that case the pruning is
  * done with no null_count==0 optimization so restoring the disjuncts is
- * no needed.
+ * not needed.
  */
 void classic_parse(Sentence sent, Parse_Options opts)
 {
@@ -279,27 +279,13 @@ void classic_parse(Sentence sent, Parse_Options opts)
 
 	Tracon_sharing *ts_pruning = pack_sentence_for_pruning(sent);
 	print_time(opts, "Encode connectors for pruning");
-	if (NULL == ts_pruning)
-	{
-		/* The sentence is considered too short for packing. */
-		if (one_step_parse)
-		{
-			/* Since no disjunct packing has been done, we will not be able
-			 * to restore them in case a parsing with null_count>0 is
-			 * needed.  Hence we must not optimize for null_count==0. */
-			needed_prune_level = 1;
-		}
-	}
-	else
-	{
-		free_sentence_disjuncts(sent);
+	free_sentence_disjuncts(sent);
 
-		if (one_step_parse)
-		{
-			/* Save the disjuncts in case we need to parse with null_count>0. */
-			disjuncts_copy = alloca(sent->length * sizeof(Disjunct *));
-			saved_memblock = save_disjuncts(sent, ts_pruning, disjuncts_copy);
-		}
+	if (one_step_parse)
+	{
+		/* Save the disjuncts in case we need to parse with null_count>0. */
+		disjuncts_copy = alloca(sent->length * sizeof(Disjunct *));
+		saved_memblock = save_disjuncts(sent, ts_pruning, disjuncts_copy);
 	}
 
 	for (int nl = min_null_count; nl <= max_null_count; nl++)
@@ -386,12 +372,16 @@ void classic_parse(Sentence sent, Parse_Options opts)
 		post_process_lkgs(sent, opts);
 
 		if (sent->num_valid_linkages > 0) break;
+
 		if (verbosity >= D_USER_INFO)
 		{
-			if (sent->num_linkages_post_processed > 0)
-				prt_error("Info: All linkages had P.P. violations.\n"
-				        "Consider to increase the linkage limit.\n"
-				        "At the command line, use !limit\n");
+			if ((sent->num_valid_linkages == 0) &&
+				 (sent->num_linkages_post_processed > 0) &&
+				 ((int)opts->linkage_limit < sent->num_linkages_found))
+				prt_error("Info: All examined linkages (%zu) had P.P. violations.\n"
+						  "Consider increasing the linkage limit.\n"
+						  "At the command line, use !limit\n",
+						  sent->num_linkages_post_processed);
 		}
 
 		if ((0 == nl) && (0 < max_null_count) && verbosity > 0)
