@@ -47,7 +47,6 @@ typedef struct
 #ifdef DEBUG
 static void print_Tconnector_list(Tconnector * e);
 static void print_clause_list(Clause * c);
-static void print_connector_list(Connector * e);
 #endif
 
 #if BUILD_DISJUNCTS_FREE_INETERMEDIATE_MEMOEY /* Undefined - CPU overhead. */
@@ -254,22 +253,32 @@ static Clause * build_clause(Exp *e, clause_context *ct)
  * string is the print name of word that generated this disjunct.
  */
 static Disjunct *
-build_disjunct(Clause * cl, const char * string, double cost_cutoff,
-               Parse_Options opts)
+build_disjunct(Sentence sent, Clause * cl, const char * string,
+               double cost_cutoff, Parse_Options opts)
 {
 	Disjunct *dis, *ndis;
+	Pool_desc *connector_pool = NULL;
+
 	dis = NULL;
 	for (; cl != NULL; cl = cl->next)
 	{
 		if (cl->maxcost <= cost_cutoff)
 		{
-			ndis = (Disjunct *) xalloc(sizeof(Disjunct));
+			if (NULL == sent) /* For the SAT-parser, until fixed. */
+			{
+				ndis = xalloc(sizeof(Disjunct));
+			}
+			else
+			{
+				ndis = pool_alloc(sent->Disjunct_pool);
+				connector_pool = sent->Connector_pool;
+			}
 			ndis->left = ndis->right = NULL;
 
 			/* Build a list of connectors from the Tconnectors. */
 			for (Tconnector *t = cl->c; t != NULL; t = t->next)
 			{
-				Connector *n = connector_new(t->e->u.condesc, opts);
+				Connector *n = connector_new(connector_pool, t->e->u.condesc, opts);
 				Connector **loc = ('-' == t->e->dir) ? &ndis->left : &ndis->right;
 
 				n->multi = t->e->multi;
@@ -286,7 +295,7 @@ build_disjunct(Clause * cl, const char * string, double cost_cutoff,
 	return dis;
 }
 
-Disjunct * build_disjuncts_for_exp(Exp* exp, const char *word,
+Disjunct * build_disjuncts_for_exp(Sentence sent, Exp* exp, const char *word,
                                    double cost_cutoff, Parse_Options opts)
 {
 	Clause *c ;
@@ -304,7 +313,7 @@ Disjunct * build_disjuncts_for_exp(Exp* exp, const char *word,
 	// print_expression(exp);  printf("\n");
 	c = build_clause(exp, &ct);
 	// print_clause_list(c);
-	dis = build_disjunct(c, word, cost_cutoff, opts);
+	dis = build_disjunct(sent, c, word, cost_cutoff, opts);
 	// print_disjunct_list(dis);
 	pool_delete(ct.Tconnector_pool);
 	pool_delete(ct.Clause_pool);
@@ -335,27 +344,6 @@ GNUC_UNUSED static void print_clause_list(Clause * c)
 	}
 }
 
-static void print_connector_list(Connector * e)
-{
-	for (;e != NULL; e=e->next)
-	{
-		printf("%s", connector_string(e));
-		if (e->next != NULL) printf(" ");
-	}
-}
-
-GNUC_UNUSED static void print_disjunct_list(Disjunct * dj)
-{
-	for (;dj != NULL; dj=dj->next) {
-		printf("%10s: ", dj->word_string);
-		printf("(%f) ", dj->cost);
-		print_connector_list(dj->left);
-		printf(" <--> ");
-		print_connector_list(dj->right);
-		printf("\n");
-	}
-}
-
 /* There is a much better print_expression elsewhere
  * This one is for low-level debug. */
 GNUC_UNUSED void prt_exp(Exp *e, int i)
