@@ -11,7 +11,7 @@ extern "C" {
 #include "utilities.h"
 }
 
-#define D_IC 6
+#define D_IC 8
 void WordTag::insert_connectors(Exp* exp, int& dfs_position,
                                 bool& leading_right, bool& leading_left,
                                 std::vector<int>& eps_right,
@@ -22,10 +22,16 @@ void WordTag::insert_connectors(Exp* exp, int& dfs_position,
   double cost = parent_cost + exp->cost;
 
 #ifdef DEBUG
-  if (0 && verbosity_level(+D_IC)) { // Extreme debug
-    printf("Expression type %d for Word%d, var %s:\n", exp->type, _word, var);
-    printf("parent_exp: "); print_expression(parent_exp);
-    printf("exp: "); print_expression(exp);
+  if (0 && verbosity_level(+D_IC)) // Extreme debug
+  //if (_word == 2)
+  {
+    const char*type =
+      ((const char *[]) {"OR_type", "AND_type", "CONNECTOR_type"}) [exp->type-1];
+    printf("Expression type %s for Word%d, var %s:\n", type, _word, var);
+    //printf("parent_exp: "); print_expression(parent_exp);
+    printf("exp(%s) e=%.2f pc=%.2f ", word_xnode->string,exp->cost, parent_cost);
+    print_expression(exp);
+    if (exp->cost > 0 || root) prt_exp_mem(exp, 0);
   }
 #endif
 
@@ -38,7 +44,7 @@ void WordTag::insert_connectors(Exp* exp, int& dfs_position,
       _dir.push_back('+');
       _right_connectors.push_back(
            PositionConnector(parent_exp, exp, '+', _word, dfs_position,
-                             cost, leading_right, false,
+                             cost, parent_cost, leading_right, false,
                              eps_right, eps_left, word_xnode, _opts));
       leading_right = false;
       break;
@@ -47,7 +53,7 @@ void WordTag::insert_connectors(Exp* exp, int& dfs_position,
       _dir.push_back('-');
       _left_connectors.push_back(
            PositionConnector(parent_exp, exp, '-', _word, dfs_position,
-                             cost, false, leading_left,
+                             cost, parent_cost, false, leading_left,
                              eps_right, eps_left, word_xnode, _opts));
       leading_left = false;
       break;
@@ -57,8 +63,14 @@ void WordTag::insert_connectors(Exp* exp, int& dfs_position,
   } else if (exp->type == AND_type) {
     if (exp->u.l == NULL) {
       /* zeroary and */
+      if (cost != 0)
+      {
+        lgdebug(+D_IC, "EmptyConnector var=%s(%d) cost %.2f pcost %.2f\n",
+                var, _variables->string(var), cost, parent_cost);
+        _empty_connectors.push_back(EmptyConnector(_variables->string(var),cost));
+      }
     } else
-      if (exp->u.l != NULL && exp->u.l->next == NULL) {
+      if (exp->u.l->next == NULL) {
         /* unary and - skip */
         insert_connectors(exp->u.l->e, dfs_position, leading_right,
              leading_left, eps_right, eps_left, var, root, cost, parent_exp, word_xnode);
@@ -79,17 +91,16 @@ void WordTag::insert_connectors(Exp* exp, int& dfs_position,
           *s++ = 'c';
           fast_sprintf(s, i);
 
+          double and_cost = (i == 0) ? cost : 0;
           insert_connectors(l->e, dfs_position, leading_right, leading_left,
-                eps_right, eps_left, new_var, false, cost, parent_exp, word_xnode);
+                eps_right, eps_left, new_var, false, and_cost, parent_exp, word_xnode);
 
-#ifdef POWER_PRUNE_CONNECTORS
           if (leading_right) {
             eps_right.push_back(_variables->epsilon(new_var, '+'));
           }
           if (leading_left) {
             eps_left.push_back(_variables->epsilon(new_var, '-'));
           }
-#endif
         }
       }
   } else if (exp->type == OR_type) {
@@ -129,20 +140,9 @@ void WordTag::insert_connectors(Exp* exp, int& dfs_position,
         lgdebug(+D_IC, "Word%d: var: %s; exp%d=%p; X_node: %s\n",
                 _word, var, i, l, word_xnode ? word_xnode->word->subword : "NULL X_node");
         assert(word_xnode != NULL, "NULL X_node for var %s", new_var);
-        if (root && parent_exp == NULL && l->e != word_xnode->exp) {
-          E_list *we = NULL;
+        if ((NULL != word_xnode->next) && (l->e == word_xnode->next->exp))
+          word_xnode = word_xnode->next;
 
-          if (word_xnode->exp->type == OR_type) {
-            for (we = word_xnode->exp->u.l; we != NULL; we = we-> next) {
-              if (l->e == we->e)
-                break;
-            }
-          }
-          if (we == NULL && word_xnode->next != NULL) {
-            lgdebug(+D_IC, "Next word_xnode for word %d is needed\n", _word);
-            word_xnode = word_xnode->next;
-          }
-        }
         insert_connectors(l->e, dfs_position, lr, ll, er, el, new_var, false, cost, l->e, word_xnode);
 
         if (lr)
