@@ -5,6 +5,11 @@
 #include <map>
 #include <set>
 
+#include "variables.hpp"
+
+#undef assert
+#include "lg_assert.h"
+
 extern "C" {
 #include "connectors.h"
 #include "dict-common/dict-common.h"
@@ -12,15 +17,13 @@ extern "C" {
 #include "tokenize/wordgraph.h"         // in_same_alternative()
 };
 
-#include "variables.hpp"
-
 struct PositionConnector
 {
   PositionConnector(Exp* pe, Exp* e, char d, int w, int p,
-                    double pcst, bool lr, bool ll,
+                    double cst, double pcst, bool lr, bool ll,
                     const std::vector<int>& er, const std::vector<int>& el, const X_node *w_xnode, Parse_Options opts)
     : exp(pe), dir(d), word(w), position(p),
-      cost(e->cost), parent_cost(pcst),
+      cost(cst), parent_cost(pcst),
       leading_right(lr), leading_left(ll),
       eps_right(er), eps_left(el), word_xnode(w_xnode)
   {
@@ -84,6 +87,19 @@ struct PositionConnector
 
 };
 
+/*
+ * Record the SAT variable and cost of costly-null expressions.
+ * Their cost is recovered (in sat_extract_links()) if
+ * they happen to reside on a participating disjunct.
+ */
+struct EmptyConnector {
+  EmptyConnector(int var, double cst)
+    : ec_var(var), ec_cost(cst)
+  {
+  }
+  int ec_var;
+  double ec_cost;
+};
 
 // XXX TODO: Hash connectors for faster matching
 
@@ -92,6 +108,7 @@ class WordTag
 private:
   std::vector<PositionConnector> _left_connectors;
   std::vector<PositionConnector> _right_connectors;
+  std::vector<EmptyConnector> _empty_connectors;
 
   std::vector<char> _dir;
   std::vector<int> _position;
@@ -110,14 +127,10 @@ private:
   }
 
 public:
-  WordTag(int word, const char* name, Variables* variables, Sentence sent, Parse_Options opts)
+  WordTag(int word, Variables* variables, Sentence sent, Parse_Options opts)
     : _word(word), _variables(variables), _sent(sent), _opts(opts) {
     _match_possible.resize(_sent->length);
 
-    // The SAT word variables are set to be equal to the word numbers.
-    Var var = _variables->string(name);
-    assert(word == var);
-
     verbosity = opts->verbosity;
     debug = opts->debug;
     test = opts->test;
@@ -137,6 +150,10 @@ public:
     return _right_connectors;
   }
 
+  const std::vector<EmptyConnector>& get_empty_connectors() const {
+    return _empty_connectors;
+  }
+
   PositionConnector* get(int dfs_position)
   {
     switch (_dir[dfs_position - 1]) {
