@@ -51,30 +51,53 @@ char *safe_strdup(const char *u)
 	return NULL;
 }
 
-/**
- * Copies as much of v into u as it can assuming u is of size usize
- * guaranteed to terminate u with a '\0'.
- */
-void safe_strcpy(char *u, const char * v, size_t usize)
-{
-	strncpy(u, v, usize-1);
-	u[usize-1] = '\0';
-}
-
 /**
  * A version of strlcpy, for those systems that don't have it.
  */
-size_t lg_strlcpy(char * dest, const char *src, size_t size)
+/*	$OpenBSD: strlcpy.c,v 1.12 2015/01/15 03:54:12 millert Exp $	*/
+/*
+ * Copyright (c) 1998, 2015 Todd C. Miller <Todd.Miller@courtesan.com>
+ *
+ * Permission to use, copy, modify, and distribute this software for any
+ * purpose with or without fee is hereby granted, provided that the above
+ * copyright notice and this permission notice appear in all copies.
+ *
+ * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
+ * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
+ * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
+ * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
+ * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
+ * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
+ * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
+ */
+/*
+ * Copy string src to buffer dst of size dsize.  At most dsize-1
+ * chars will be copied.  Always NUL terminates (unless dsize == 0).
+ * Returns strlen(src); if retval >= dsize, truncation occurred.
+ */
+size_t
+lg_strlcpy(char * restrict dst, const char * restrict src, size_t dsize)
 {
-	size_t i=0;
-	while ((i<size) && (src[i] != 0x0))
-	{
-		dest[i] = src[i];
-		i++;
+	const char *osrc = src;
+	size_t nleft = dsize;
+
+	/* Copy as many bytes as will fit. */
+	if (nleft != 0) {
+		while (--nleft != 0) {
+			if ((*dst++ = *src++) == '\0')
+				break;
+		}
 	}
-	if (i < size) { dest[i] = 0x0; size = i; }
-	else if (0 < size) { size --; dest[size] = 0x0;}
-	return size;
+
+	/* Not enough room in dst, add NUL and traverse rest of src. */
+	if (nleft == 0) {
+		if (dsize != 0)
+			*dst = '\0';      /* NUL-terminate dst */
+		while (*src++)
+			;
+	}
+
+	return(src - osrc - 1); /* count does not include NUL */
 }
 
 /**
@@ -237,7 +260,7 @@ void downcase_utf8_str(char *to, const char * from, size_t usize, locale_t local
 
 	from += nbh;
 	to += nbl;
-	safe_strcpy(to, from, usize-nbl);
+	lg_strlcpy(to, from, usize-nbl);
 }
 
 #if 0
@@ -280,7 +303,7 @@ void upcase_utf8_str(char *to, const char * from, size_t usize, locale_t locale)
 
 	from += nbh;
 	to += nbl;
-	safe_strcpy(to, from, usize-nbl);
+	lg_strlcpy(to, from, usize-nbl);
 }
 #endif
 
