@@ -37,7 +37,6 @@ static struct
 	int allow_null;
 	int use_cluster_disjuncts;
 	int use_sat_solver;
-	int use_viterbi;
 	int echo_on;
 	Cost_Model_type cost_model;
 	double max_cost;
@@ -98,9 +97,6 @@ Switch default_switches[] =
 	{"verbosity",  Int,  "Level of detail in output",       &local.verbosity},
 	{"debug",      String, "Comma-separated function names to debug", &local.debug},
 	{"test",       String, "Comma-separated test features", &local.test},
-#ifdef USE_VITERBI
-	{"viterbi",    Bool, "Use Viterbi-based parser",        &local.use_viterbi},
-#endif
 	{"walls",      Bool, "Display wall words",              &local.display_walls},
 	{"width",      Int,  "The width of the display",        &local.screen_width},
 	{"wordgraph",  Int,  "Display sentence word-graph",     &local.display_wordgraph},
@@ -397,7 +393,7 @@ static FILE *open_help_file(int verbosity)
 	{
 		if (NULL != help_file)
 			continue; /* until get_next_locale() returns NULL */
-		strncpy(ll_pos, ll, HELPFILE_LANG_TEMPLATE_SIZE);
+		memcpy(ll_pos, ll, HELPFILE_LANG_TEMPLATE_SIZE);
 		help_file = linkgrammar_open_data_file(help_filename);
 	}
 
@@ -549,6 +545,25 @@ static void display_help(const Switch *sp, Command_Options *copts)
 	}
 }
 
+#define URL_LINE "%-20s %s\n"
+void print_url_info(void)
+{
+	printf("\n");
+#ifdef OVERVIEW
+	printf(URL_LINE, "Overview:", OVERVIEW);
+#endif /* OVERVIEW */
+#ifdef PACKAGE_URL
+	printf(URL_LINE, "Home page:", PACKAGE_URL);
+	printf(URL_LINE, "Documentation:", PACKAGE_URL"/dict/");
+#endif /* PACKAGE_URL */
+#ifdef DISCUSSION_GROUP
+	printf(URL_LINE, "Discussion group:", DISCUSSION_GROUP);
+#endif /* DISCUSSION_GROUP */
+#ifdef PACKAGE_BUGREPORT
+	printf(URL_LINE, "Report bugs to:" , PACKAGE_BUGREPORT"/issues");
+#endif /* PACKAGE_BUGREPORT */
+}
+
 static int help_cmd(const Switch *uc, int n)
 {
 	printf("Special commands always begin with \"!\".  Command and variable names\n");
@@ -570,6 +585,8 @@ static int help_cmd(const Switch *uc, int n)
 	printf(" !<var>          Toggle the specified Boolean variable.\n");
 	printf(" !<var>=<val>    Assign that value to that variable.\n");
 
+	print_url_info();
+
 	return 'c';
 }
 
@@ -789,7 +806,7 @@ static int x_issue_special_command(char * line, Command_Options *copts, Dictiona
 		{
 			char *err;
 			double val = strtod(y, &err);
-			if (('\0' == *y) ||('\0' != *err))
+			if (('\0' == *y) || ('\0' != *err))
 			{
 				prt_error("Error: Invalid value %s for variable \"%s\". Type \"!help\" or \"!variables\"\n", y, as[j].string);
 				return -1;
@@ -831,11 +848,12 @@ static int x_issue_special_command(char * line, Command_Options *copts, Dictiona
 
 	if (0 < count)
 	{
-		prt_error("Error: Variable \"%s\" requires a value.  Try \"!help\".\n", as[j].string);
+		prt_error("Error: Variable \"%s\" requires a value.  Try \"!help %s\".\n",
+		          as[j].string, as[j].string);
 		return -1;
 	}
 
-	prt_error("Error: I can't interpret \"%s\" as a command.  Try \"!help\".\n", line);
+	prt_error("Error: I can't interpret \"%s\" as a command.  Try \"!help\" or \"!variables\".\n", line);
 	return -1;
 }
 
@@ -856,9 +874,6 @@ static void put_opts_in_local_vars(Command_Options* copts)
 	local.max_cost = parse_options_get_disjunct_cost(opts);
 	local.use_cluster_disjuncts = parse_options_get_use_cluster_disjuncts(opts);
 	local.use_sat_solver = parse_options_get_use_sat_parser(opts);
-#ifdef USE_VITERBI
-	local.use_viterbi = parse_options_get_use_viterbi(opts);
-#endif
 
 	local.screen_width = (int)copts->screen_width;
 	local.echo_on = copts->echo_on;
@@ -898,9 +913,6 @@ static void put_local_vars_in_opts(Command_Options* copts)
 	parse_options_set_use_cluster_disjuncts(opts, local.use_cluster_disjuncts);
 #ifdef USE_SAT_SOLVER
 	parse_options_set_use_sat_parser(opts, local.use_sat_solver);
-#endif
-#ifdef USE_VITERBI
-	parse_options_set_use_viterbi(opts, local.use_viterbi);
 #endif
 	parse_options_set_display_morphology(opts, local.display_morphology);
 
