@@ -64,9 +64,6 @@
 #include "parser-utilities.h"
 #include "command-line.h"
 #include "lg_readline.h"
-#ifdef USE_VITERBI
-#include "../viterbi/viterbi.h"
-#endif /* USE_VITERBI */
 
 #define DISPLAY_MAX 1024
 
@@ -92,10 +89,6 @@ static char * get_terminal_line(char *input_string, FILE *in, FILE *out)
 	const char *prompt = (0 == verbosity)? "" : "linkparser> ";
 
 #ifdef HAVE_EDITLINE
-	#ifdef _WIN32
-		#error __FILE__ ": Cannot use HAVE_EDITLINE "
-		                "(the console already has line editing and history)."
-	#endif /* _WIN32 */
 	pline = lg_readline(prompt);
 #else
 	fprintf(out, "%s", prompt);
@@ -261,6 +254,43 @@ static void print_parse_statistics(Sentence sent, Parse_Options opts,
 	}
 }
 
+/**
+ * Check whether the given feature is enabled. It is considered
+ * enabled if it is found in the comma delimited list of features.
+ * This list, if not empty, has a leading and a trailing commas.
+ * Return NULL if not enabled, else ",". If the feature appears
+ * as "feature:param", return a pointer to the ":".
+ *
+ * This function is similar to feature_enabled() of the library (which
+ * is not exported) besides not including filename matching.
+ */
+static const char *test_enabled(const char *feature, const char *test_name)
+{
+
+	if ('\0' == feature[0]) return NULL;
+	size_t len = strlen(test_name);
+	char *buff = malloc(len + 2 + 1); /* leading comma + comma/colon + NUL */
+	const char *r = NULL;
+
+	buff[0] = ',';
+	strcpy(buff+1, test_name);
+	strcat(buff, ",");
+
+	if (NULL != strstr(feature, buff))
+	{
+		r = ",";
+	}
+	else
+	{
+		buff[len+1] = ':'; /* check for "feature:param" */
+		if (NULL != strstr(feature, buff))
+			r = strstr(feature, buff) + len + 1;
+	}
+
+	free(buff);
+	return r;
+}
+
 /**
  * Check for the auto-next-linkage test request (for LG code development).
  * It is given using the special command: test=auto-next-linkage[:display_max]
@@ -272,12 +302,13 @@ static void print_parse_statistics(Sentence sent, Parse_Options opts,
  */
 static int auto_next_linkage_test(const char *test_opt)
 {
-	char auto_next_linkage_str[] = ",auto-next-linkage";
-	char *auto_next_linkage_pos = strstr(test_opt, auto_next_linkage_str);
-	int max_display;
+	const char *auto_next_linkage_pos =
+		test_enabled(test_opt, "auto-next-linkage");
+	int max_display = 0;
 
 	if (auto_next_linkage_pos == NULL) return 0;
-	max_display = atoi(auto_next_linkage_pos + sizeof(auto_next_linkage_str));
+	if (':' == auto_next_linkage_pos[0])
+		max_display = atoi(auto_next_linkage_pos + 1);
 	if (max_display != 0) return max_display;
 	return DISPLAY_MAX;
 }
@@ -448,7 +479,7 @@ static void batch_process_some_linkages(Label label,
 	}
 	else
 	{
-		if (strstr(test, ",batch_print_parse_statistics,"))
+		if (test_enabled(test, "batch-print-parse-statistics"))
 		{
 			print_parse_statistics(sent, opts, copts);
 		}
@@ -510,17 +541,41 @@ static void restore_stdio(FILE *from, int origfd)
 }
 #endif
 
-static void print_usage(FILE *out, char *str, int exit_value)
+/**
+ * Find the basename of the given file name.
+ * The last component that starts with '\\' or '\'
+ * (whichever is last) is returned.
+ * On POSIX systems it can be confused if the program name
+ * contains '\\' characters, but we don't care.
+ */
+static const char *fbasename(const char *fpath)
 {
-	Command_Options *copts;
+	const char *progf, *progb;
+
+	if ((NULL == fpath) || ('\0' == fpath[0])) return "(null)";
+
+	progf = strrchr(fpath, '/');
+	if (NULL == progf)
+		progb = strrchr(fpath, '\\');
+	else
+		progb = strchr(progf, '\\');
+
+	if (NULL != progb) return progb + 1;
+	if (NULL == progf) return fpath;
+	return progf + 1;
+}
+
+static void print_usage(FILE *out, char *argv0, Command_Options *copts, int exit_value)
+{
+
 	fprintf(out, "Usage: %s [language|dictionary location]\n"
 			 "                   [-<special \"!\" command>]\n"
-			 "                   [--version]\n", str);
+			 "                   [--version]\n", fbasename(argv0));
 
 	fprintf(out, "\nSpecial commands are:\n");
-	copts = command_options_create();
-	divert_stdio(stdout, stderr);
+	if (stdout != out) divert_stdio(stdout, out);
 	issue_special_command("var", copts, NULL);
+	if (out == stdout) print_url_info(); /* don't print it for errors */
 	exit(exit_value);
 }
 
@@ -567,7 +622,7 @@ fail:
 	/* printf("cols %i\n", ws.ws_col); */
 
 	/* Set the screen width only if the returned value seems
-	 * rational: its positive and not insanely tiny.
+	 * rational: it's positive and not insanely tiny.
 	 */
 	if ((10 < ws.ws_col) && (16123 > ws.ws_col))
 	{
@@ -581,7 +636,7 @@ int main(int argc, char * argv[])
 	FILE            *input_fh = stdin;
 	Dictionary      dict;
 	const char     *language = NULL;
-	int             num_linkages, i;
+	int             num_linkages;
 	Label           label = NO_LABEL;
 	Command_Options *copts;
 	Parse_Options   opts;
@@ -597,6 +652,20 @@ int main(int argc, char * argv[])
 	const char *ostype = getenv("OSTYPE");
 	if ((NULL != ostype) && (0 == strcmp(ostype, "cygwin")))
 		running_under_cygwin = true;
+
+	/* argv encoding is in the current locale. */
+	argv = argv2utf8(argc);
+	if (NULL == argv)
+	{
+		prt_error("Fatal error: Unable to parse command line\n");
+		exit(-1);
+	}
+
+#ifdef _MSC_VER
+	_set_printf_count_output(1); /* enable %n support for display_1line_help()*/
+#endif /* _MSC_VER */
+
+	win32_set_utf8_output();
 #endif /* _WIN32 */
 
 #if LATER
@@ -609,28 +678,6 @@ int main(int argc, char * argv[])
 	sigaction (SIGWINCH, &winch_act, NULL);
 #endif
 
-	i = 1;
-	if ((argc > 1) && (argv[1][0] != '-')) {
-		/* the dictionary is the first argument if it doesn't begin with "-" */
-		language = argv[1];
-		i++;
-	}
-
-	if ((i < argc) && strcmp("--help", argv[i]) == 0)
-	{
-		print_usage(stdout, argv[0], 0);
-	}
-
-	for (; i<argc; i++)
-	{
-		if (argv[i][0] == '-' && strcmp("--version", argv[i]) == 0)
-		{
-			printf("Version: %s\n", linkgrammar_get_version());
-			printf("%s\n", linkgrammar_get_configuration());
-			exit(0);
-		}
-	}
-
 	copts = command_options_create();
 	if (copts == NULL || copts->panic_opts == NULL)
 	{
@@ -648,29 +695,46 @@ int main(int argc, char * argv[])
 	parse_options_set_max_null_count(opts, 0);
 	parse_options_set_short_length(opts, 16);
 	parse_options_set_islands_ok(opts, false);
+	parse_options_set_display_morphology(opts, false);
 
 	save_default_opts(copts); /* Options so far are the defaults */
 
+	if ((argc > 1) && (argv[1][0] != '-')) {
+		/* The dictionary is the first argument if it doesn't begin with "-" */
+		language = argv[1];
+	}
+
+	for (int i = 1; i < argc; i++)
+	{
+		if (strcmp("--help", argv[i]) == 0)
+		{
+			print_usage(stdout, argv[0], copts, 0);
+		}
+
+		if (strcmp("--version", argv[i]) == 0)
+		{
+			printf("Version: %s\n", linkgrammar_get_version());
+			printf("%s\n", linkgrammar_get_configuration());
+			exit(0);
+		}
+	}
+
 	/* Process command line variable-setting commands (only). */
-	for (i = 1; i < argc; i++)
+	for (int i = 1; i < argc; i++)
 	{
 		if (argv[i][0] == '-')
 		{
 			const char *var = argv[i] + ((argv[i][1] != '-') ? 1 : 2);
 			if ((var[0] != '!') && (0 > issue_special_command(var, copts, NULL)))
-				print_usage(stderr, argv[0], -1);
+				print_usage(stderr, argv[0], copts, -1);
 		}
 		else if (i != 1)
 		{
 			prt_error("Fatal error: Unknown argument '%s'.\n", argv[i]);
-			print_usage(stderr, argv[0], -1);
+			print_usage(stderr, argv[0], copts, -1);
 		}
 	}
 
-#ifdef _WIN32
-	win32_set_utf8_output();
-#endif /* _WIN32 */
-
 	if (language && *language)
 	{
 		dict = dictionary_create_lang(language);
@@ -691,12 +755,12 @@ int main(int argc, char * argv[])
 	}
 
 	/* Process the command line '!' commands */
-	for (i = 1; i<argc; i++)
+	for (int i = 1; i < argc; i++)
 	{
 		if ((argv[i][0] == '-') && (argv[i][1] == '!'))
 		{
 			if (0 > issue_special_command(argv[i]+1, copts, dict))
-				print_usage(stderr, argv[0], -1);
+				print_usage(stderr, argv[0], copts, -1);
 		}
 	}
 
@@ -752,7 +816,7 @@ int main(int argc, char * argv[])
 		if ('c' == command) continue; /* It was another command */
 		if (-1 == command) continue;  /* It was a bad command */
 
-		/* We have to handle the !file command inline; its too hairy
+		/* We have to handle the !file command inline; it's too hairy
 		 * otherwise ... */
 		if ('f' == command)
 		{
@@ -806,7 +870,7 @@ int main(int argc, char * argv[])
 			printf("%s\n", input_string);
 		}
 
-		if (copts->batch_mode)
+		if (copts->batch_mode || auto_next_linkage_test(test))
 		{
 			label = strip_off_label(input_string);
 		}
@@ -819,173 +883,176 @@ int main(int argc, char * argv[])
 		else
 			parse_options_set_perform_pp_prune(opts, true);
 
-#ifdef USE_VITERBI
-		/* Compile-time optional, for now, since it don't work yet. */
-		if (parse_options_get_use_viterbi(opts))
+		sent = sentence_create(input_string, dict);
+
+		if (sentence_split(sent, opts) < 0)
 		{
-			viterbi_parse(input_string, dict);
-		}
-		else
-#endif /* USE_VITERBI */
-		{
-			sent = sentence_create(input_string, dict);
-
-			/* First parse with cost 0 or 1 and no null links */
-			// parse_options_set_disjunct_cost(opts, 2.7);
-			parse_options_set_min_null_count(opts, 0);
-			parse_options_set_max_null_count(opts, 0);
-			parse_options_reset_resources(opts);
-
-			num_linkages = sentence_parse(sent, opts);
-
-			/* num_linkages is negative only on a hard-error;
-			 * typically, due to a zero-length sentence.  */
-			if (num_linkages < 0)
-			{
-				sentence_delete(sent);
-				sent = NULL;
-				continue;
-			}
-
-			if (0 != copts->display_wordgraph)
-			{
-				const char *wg_display_flags = ""; /* default flags */
-				switch (copts->display_wordgraph)
-				{
-					case 1:     /* default flags */
-						break;
-					case 2:     /* subgraphs with a legend */
-						wg_display_flags = "sl";
-						break;
-					case 3:
-						{
-							/* Use esoteric flags from the test user variable. */
-							const char wg[] = ",wg";
-							const char *s = strstr(test, wg);
-							if (NULL != s) wg_display_flags = s+2;
-						}
-						break;
-					default:
-						prt_error("Warning: wordgraph=%d: Unknown value, using 1\n",
-						          copts->display_wordgraph);
-						copts->display_wordgraph = 1;
-				}
-				sentence_display_wordgraph(sent, wg_display_flags);
-			}
-#if 0
-			/* Try again, this time omitting the requirement for
-			 * definite articles, etc. This should allow for the parsing
-			 * of newspaper headlines and other clipped speech.
-			 *
-			 * XXX Unfortunately, this also allows for the parsing of
-			 * all sorts of ungrammatical sentences which should not
-			 * parse, and leads to bad parses of many other unparsable
-			 * but otherwise grammatical sentences.  Thus, this trick
-			 * pretty much fails; we leave it here to document the
-			 * experiment.
-			 */
-			if (num_linkages == 0)
-			{
-				parse_options_set_disjunct_cost(opts, 4.5);
-				num_linkages = sentence_parse(sent, opts);
-				if (num_linkages < 0) continue;
-			}
-#endif /* 0 */
-
-			/* Try using a larger list of disjuncts */
-			/* XXX FIXME: the lg_expand_disjunct_list() routine is not
-			 * currently a part of the public API; it should be made so,
-			 * or this expansion idea should be abandoned... not sure which.
-			 */
-			if ((num_linkages == 0) && parse_options_get_use_cluster_disjuncts(opts))
-			{
-				int expanded;
-				if (verbosity > 0) fprintf(stdout, "No standard linkages, expanding disjunct set.\n");
-				parse_options_set_disjunct_cost(opts, 3.9);
-				expanded = lg_expand_disjunct_list(sent);
-				if (expanded)
-				{
-					num_linkages = sentence_parse(sent, opts);
-				}
-				if (0 < num_linkages) printf("Got One !!!!!!!!!!!!!!!!!\n");
-			}
-
-			/* If asked to show bad linkages, then show them. */
-			if ((num_linkages == 0) && (!copts->batch_mode))
-			{
-				if (copts->display_bad)
-				{
-					num_linkages = sentence_num_linkages_found(sent);
-				}
-			}
-
-			/* Now parse with null links */
-			if (num_linkages == 0 && !copts->batch_mode)
-			{
-				if (verbosity > 0) fprintf(stdout, "No complete linkages found.\n");
-
-				if (copts->allow_null)
-				{
-					/* XXX should use expanded disjunct list here too */
-					parse_options_set_min_null_count(opts, 1);
-					parse_options_set_max_null_count(opts, sentence_length(sent));
-					num_linkages = sentence_parse(sent, opts);
-				}
-			}
-
-			if (verbosity > 0)
-			{
-				if (parse_options_timer_expired(opts))
-					fprintf(stdout, "Timer is expired!\n");
-
-				if (parse_options_memory_exhausted(opts))
-					fprintf(stdout, "Memory is exhausted!\n");
-			}
-
-			if ((num_linkages == 0) &&
-				copts->panic_mode &&
-				parse_options_resources_exhausted(opts))
-			{
-				/* print_total_time(opts); */
-				batch_errors++;
-				if (verbosity > 0) fprintf(stdout, "Entering \"panic\" mode...\n");
-				/* If the parser used was the SAT solver, set the panic parser to
-				 * it too.
-				 * FIXME? Currently, the SAT solver code is not too useful in
-				 * panic mode since it doesn't handle parsing with null words, so
-				 * using the regular parser in that case could be beneficial.
-				 * However, this currently causes a crash due to a memory
-				 * management mess. */
-				parse_options_set_use_sat_parser(copts->panic_opts,
-					parse_options_get_use_sat_parser(opts));
-				parse_options_reset_resources(copts->panic_opts);
-				parse_options_set_verbosity(copts->panic_opts, verbosity);
-				(void)sentence_parse(sent, copts->panic_opts);
-				if (verbosity > 0)
-				{
-					if (parse_options_timer_expired(copts->panic_opts))
-						fprintf(stdout, "Panic timer is expired!\n");
-				}
-			}
-
-			/* print_total_time(opts); */
-
-			const char *rc = "";
-			if (copts->batch_mode)
-			{
-				batch_process_some_linkages(label, sent, copts);
-			}
-			else
-			{
-				rc = process_some_linkages(input_fh, sent, copts);
-			}
-
-			fflush(stdout);
 			sentence_delete(sent);
 			sent = NULL;
-
-			if ((NULL == rc) && (input_fh == stdin)) break;
+			continue;
 		}
+
+		if (0 != copts->display_wordgraph)
+		{
+			const char *wg_display_flags = ""; /* default flags */
+			switch (copts->display_wordgraph)
+			{
+				case 1:     /* default flags */
+					break;
+				case 2:     /* subgraphs with a legend */
+					wg_display_flags = "sl";
+					break;
+				case 3:
+					{
+						/* Use esoteric flags from the test user variable. */
+						const char *s = test_enabled(test, "wg");
+						if ((NULL != s) && (':' == s[0])) wg_display_flags = s;
+					}
+					break;
+				default:
+					prt_error("Warning: wordgraph=%d: Unknown value, using 1\n",
+								 copts->display_wordgraph);
+					copts->display_wordgraph = 1;
+			}
+			sentence_display_wordgraph(sent, wg_display_flags);
+		}
+
+		/* First parse with the default disjunct_cost as set by the library
+		 * (currently 2.7). Usually parse here with no null links.
+		 * However, if "-test=one-step-parse" is used and we are said to
+		 * parse with null links, allow parsing here with null links too. */
+		bool one_step_parse = !copts->batch_mode && copts->allow_null &&
+		                    test_enabled(test, "one-step-parse");
+		int max_null_count = one_step_parse ? sentence_length(sent) : 0;
+
+		parse_options_set_min_null_count(opts, 0);
+		parse_options_set_max_null_count(opts, max_null_count);
+		parse_options_reset_resources(opts);
+
+		num_linkages = sentence_parse(sent, opts);
+
+		/* num_linkages is negative only on a hard-error;
+		 * typically, due to a zero-length sentence.  */
+		if (num_linkages < 0)
+		{
+			sentence_delete(sent);
+			sent = NULL;
+			continue;
+		}
+
+#if 0
+		/* Try again, this time omitting the requirement for
+		 * definite articles, etc. This should allow for the parsing
+		 * of newspaper headlines and other clipped speech.
+		 *
+		 * XXX Unfortunately, this also allows for the parsing of
+		 * all sorts of ungrammatical sentences which should not
+		 * parse, and leads to bad parses of many other unparsable
+		 * but otherwise grammatical sentences.  Thus, this trick
+		 * pretty much fails; we leave it here to document the
+		 * experiment.
+		 */
+		if (num_linkages == 0)
+		{
+			parse_options_set_disjunct_cost(opts, 4.5);
+			num_linkages = sentence_parse(sent, opts);
+			if (num_linkages < 0) continue;
+		}
+#endif /* 0 */
+
+		/* Try using a larger list of disjuncts */
+		/* XXX FIXME: the lg_expand_disjunct_list() routine is not
+		 * currently a part of the public API; it should be made so,
+		 * or this expansion idea should be abandoned... not sure which.
+		 */
+		if ((num_linkages == 0) && parse_options_get_use_cluster_disjuncts(opts))
+		{
+			int expanded;
+			if (verbosity > 0) fprintf(stdout, "No standard linkages, expanding disjunct set.\n");
+			parse_options_set_disjunct_cost(opts, 3.9);
+			expanded = lg_expand_disjunct_list(sent);
+			if (expanded)
+			{
+				num_linkages = sentence_parse(sent, opts);
+			}
+			if (0 < num_linkages) printf("Got One !!!!!!!!!!!!!!!!!\n");
+		}
+
+		/* If asked to show bad linkages, then show them. */
+		if ((num_linkages == 0) && (!copts->batch_mode))
+		{
+			if (copts->display_bad)
+			{
+				num_linkages = sentence_num_linkages_found(sent);
+			}
+		}
+
+		/* Now parse with null links */
+		if (!one_step_parse && num_linkages == 0 && !copts->batch_mode)
+		{
+			if (verbosity > 0) fprintf(stdout, "No complete linkages found.\n");
+
+			if (copts->allow_null)
+			{
+				/* XXX should use expanded disjunct list here too */
+				parse_options_set_min_null_count(opts, 1);
+				parse_options_set_max_null_count(opts, sentence_length(sent));
+				num_linkages = sentence_parse(sent, opts);
+			}
+		}
+
+		if (verbosity > 0)
+		{
+			if (parse_options_timer_expired(opts))
+				fprintf(stdout, "Timer is expired!\n");
+
+			if (parse_options_memory_exhausted(opts))
+				fprintf(stdout, "Memory is exhausted!\n");
+		}
+
+		if ((num_linkages == 0) &&
+			copts->panic_mode &&
+			parse_options_resources_exhausted(opts))
+		{
+			/* print_total_time(opts); */
+			batch_errors++;
+			if (verbosity > 0) fprintf(stdout, "Entering \"panic\" mode...\n");
+			/* If the parser used was the SAT solver, set the panic parser to
+			 * it too.
+			 * FIXME? Currently, the SAT solver code is not too useful in
+			 * panic mode since it doesn't handle parsing with null words, so
+			 * using the regular parser in that case could be beneficial.
+			 * However, this currently causes a crash due to a memory
+			 * management mess. */
+			parse_options_set_use_sat_parser(copts->panic_opts,
+				parse_options_get_use_sat_parser(opts));
+			parse_options_reset_resources(copts->panic_opts);
+			parse_options_set_verbosity(copts->panic_opts, verbosity);
+			(void)sentence_parse(sent, copts->panic_opts);
+			if (verbosity > 0)
+			{
+				if (parse_options_timer_expired(copts->panic_opts))
+					fprintf(stdout, "Panic timer is expired!\n");
+			}
+		}
+
+		if (verbosity > 1) parse_options_print_total_time(opts);
+
+		const char *rc = "";
+		if (copts->batch_mode)
+		{
+			batch_process_some_linkages(label, sent, copts);
+		}
+		else
+		{
+			rc = process_some_linkages(input_fh, sent, copts);
+		}
+
+		fflush(stdout);
+		sentence_delete(sent);
+		sent = NULL;
+
+		if ((NULL == rc) && (input_fh == stdin)) break;
 	}
 
 	if (copts->batch_mode)
